# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""recommender API utlities."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.recommender import service as recommender_service
from googlecloudsdk.api_lib.util import messages as messages_util
from googlecloudsdk.calliope import base
from googlecloudsdk.core import yaml

RECOMMENDER_API_ALPHA_VERSION = 'v1alpha2'
RECOMMENDER_API_BETA_VERSION = 'v1beta1'
RECOMMENDER_API_GA_VERSION = 'v1'


def ToCamelCase(s):
  """Converts CamelCase to camelCase."""
  return s[0].lower() + s[1:]


def ReadConfig(config_file, message_type):
  """Parses json config file.

  Args:
    config_file: file path of the config file.
    message_type: The protorpc Message type.

  Returns:
    A message of type "message_type".
  """
  config = None
  # Yaml is a superset of json, so parse json file as yaml.
  data = yaml.load_path(config_file)
  if data:
    config = messages_util.DictToMessageWithErrorCheck(data, message_type)
  return config


def GetConfigServiceFromArgs(api_version, is_insight_api):
  """Returns the config api service from the user-specified arguments.

  Args:
    api_version: API version string.
    is_insight_api: boolean value sepcify whether this is a insight api,
      otherwise will return a recommendation service api.
  """
  if is_insight_api:
    return recommender_service.ProjectsInsightTypeConfigsService(api_version)
  return recommender_service.ProjectsRecommenderConfigsService(api_version)


def GetDescribeConfigRequestFromArgs(parent_resource, is_insight_api,
                                     api_version):
  """Returns the describe request from the user-specified arguments.

  Args:
    parent_resource: resource url string, the flags are already defined in
      argparse namespace.
    is_insight_api: boolean value specifying whether this is a insight api,
      otherwise treat as a recommender service api and return related describe
      request message.
    api_version: API version string.
  """

  messages = recommender_service.RecommenderMessages(api_version)
  if is_insight_api:
    request = messages.RecommenderProjectsLocationsInsightTypesGetConfigRequest(
        name=parent_resource)
  else:
    request = messages.RecommenderProjectsLocationsRecommendersGetConfigRequest(
        name=parent_resource)
  return request


def GetApiVersion(release_track):
  """Get API version string.

  Converts API version string from release track value.

  Args:
    release_track: release_track value, can be ALPHA, BETA, GA

  Returns:
    API version string.
  """
  switcher = {
      base.ReleaseTrack.ALPHA: RECOMMENDER_API_ALPHA_VERSION,
      base.ReleaseTrack.BETA: RECOMMENDER_API_BETA_VERSION,
      base.ReleaseTrack.GA: RECOMMENDER_API_GA_VERSION,
  }
  return switcher.get(release_track, RECOMMENDER_API_ALPHA_VERSION)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for spanner instance configs create."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import textwrap
from googlecloudsdk.api_lib.spanner import instance_config_operations
from googlecloudsdk.api_lib.spanner import instance_configs
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as c_exceptions
from googlecloudsdk.command_lib.spanner import flags
from googlecloudsdk.command_lib.util.args import labels_util


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.Hidden
class Create(base.CreateCommand):
  """Create a Cloud Spanner instance config."""

  detailed_help = {
      'EXAMPLES':
          textwrap.dedent("""\
        To create a Cloud Spanner instance config, run:

          $ {command} custom-instance-config
            --display-name=custom-instance-config-name
            --base-config=nam3
            --replicas=location=us-east4,type=READ_WRITE:location=us-east4,type=READ_WRITE:location=us-east1,type=READ_WRITE:location=us-east1,type=READ_WRITE:location=us-central1,type=WITNESS

        To create a Cloud Spanner instance config based on an existing Google managed configs (nam3) and user managed configs with adding replica 'us-east4' of type 'READ_WRITE', run:

          $ {command} custom-instance-config
            --clone-config=nam3
            --add-replicas=location=us-east4,type=READ_WRITE

        To create a Cloud Spanner instance config based on an existing Google managed configs (nam3) and user managed configs with adding replica 'us-east4' of type 'READ_WRITE' and removing replica 'us-central1' of type 'READ_ONLY', run:

          $ {command} custom-instance-config
            --clone-config=nam3
            --add-replicas=location=us-east4,type=READ_WRITE
            --skip-replicas=location=us-central1,type=READ_ONLY
        """),
  }

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    parser.add_argument(
        'config',
        metavar='INSTANCE_CONFIG',
        completer=flags.InstanceConfigCompleter,
        help='Cloud Spanner instance config. The `custom-` prefix is required '
        'to avoid name conflicts with Google managed configurations.')

    parser.add_argument(
        '--display-name',
        help='The name of this instance configuration as it appears in UIs. '
        'Must specify this option if creating an instance-config with '
        '--replicas.')

    parser.add_argument(
        '--etag', help='Used for optimistic concurrency control.')

    base.ASYNC_FLAG.AddToParser(parser)
    labels_util.AddCreateLabelsFlags(parser)

    parser.add_argument(
        '--validate-only',
        action='store_true',
        default=False,
        help='Validate the create action, but don\'t actually perform it.')

    replica_help_text = """\
        The geographic placement of nodes in this instance configuration and
        their replication properties.

        *location*::: The location of the serving resources, e.g. "us-central1".

        *type*::: The type of replica.

        Items in the list are separated by ":". The allowed values and formats
        are as follows.

        *READ_ONLY*::::

        Read-only replicas only support reads (not writes). Read-only
        replicas:

          * Maintain a full copy of your data.

          * Serve reads.

          * Do not participate in voting to commit writes.

          * Are not eligible to become a leader.

        *READ_WRITE*::::

        Read-write replicas support both reads and writes. These
        replicas:

          * Maintain a full copy of your data.

          * Serve reads.

          * Can vote whether to commit a write.

          * Participate in leadership election.

          * Are eligible to become a leader.

        *WITNESS*::::

        Witness replicas don't support reads but do participate in
        voting to commit writes. Witness replicas:

          * Do not maintain a full copy of data.

          * Do not serve reads.

          * Vote whether to commit writes.

          * Participate in leader election but are not eligible to become
            leader.

        """
    clone_or_manual = parser.add_mutually_exclusive_group(required=True)
    manual_flags = clone_or_manual.add_argument_group(
        'Command-line flags to setup an instance-config replicas:')
    flags.ReplicaFlag(manual_flags, name='--replicas', text=replica_help_text)
    manual_flags.add_argument(
        '--base-config',
        required=True,
        help='The ID of the instance config, based on which this configuration '
        'is created.')

    clone_flags = clone_or_manual.add_argument_group(
        'Command-line flags to setup an instance-config using clone options:')
    clone_flags.add_argument(
        '--clone-config',
        required=True,
        metavar='INSTANCE_CONFIG',
        completer=flags.InstanceConfigCompleter,
        help='The ID of the instance config, based on which this '
        'configuration is created. The clone is an independent copy of this '
        'config. Available configurations can be found by running '
        '"gcloud spanner instance-configs list"')
    flags.ReplicaFlag(
        clone_flags,
        name='--add-replicas',
        text='Add new replicas while cloning from the source config.',
        required=False)
    flags.ReplicaFlag(
        clone_flags,
        name='--skip-replicas',
        text='Skip replicas from the source config while cloning. Each replica '
        'in the list must exist in the source config replicas list.',
        required=False)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Instance config create response.
    """
    if args.clone_config:
      # If the config exists, it's cloned, otherwise, we display the
      # error from instanceConfigs.Get.
      config = instance_configs.Get(args.clone_config)
      op = instance_configs.CreateUsingExistingConfig(args, config)
    else:
      if not args.IsSpecified('display_name'):
        raise c_exceptions.InvalidArgumentException(
            '--display-name', 'Must specify --display-name.')

      op = instance_configs.CreateUsingReplicas(args.config, args.display_name,
                                                args.base_config, args.replicas,
                                                args.validate_only, args.labels,
                                                args.etag)
    if args.async_:
      return op
    return instance_config_operations.Await(op, 'Creating instance-config')



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for error reporting."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import re

from googlecloudsdk.core.util import encoding

PARTITION_TRACEBACK_PATTERN = (
    r'(?P<stacktrace>'
    r'Traceback \(most recent call last\):\n'
    r'(?: {2}File ".*", line \d+, in .+\n {4}.+\n)+'
    r')'
    r'(?P<exception>\S.+)')

FILE_PATH_PATTERN = (
    r' {2}File "(?P<file>.*)google-cloud-sdk.*", line \d+, in .+')


def RemovePrivateInformationFromTraceback(traceback):
  """Given a stacktrace, only include Cloud SDK files in path.

  Args:
    traceback: str, the original unformatted traceback

  Returns:
    str, A new stacktrace with the private paths removed
    None, If traceback does not match traceback pattern
  """
  # Check that traceback follows pattern
  match = re.search(PARTITION_TRACEBACK_PATTERN, traceback)
  if not match:
    return None

  # Separate to individual lines
  stacktrace_list = traceback.splitlines()
  remove_path_stacktrace_list = []

  pattern_file_path = re.compile(FILE_PATH_PATTERN)

  lines_since_match = 0
  for line in stacktrace_list:
    match = pattern_file_path.match(line)
    if match:
      remove_path_stacktrace_list.append(line.replace(match.group('file'), ''))
      lines_since_match = 0
    else:
      lines_since_match += 1
      # This will only be true for the second and subsequent message lines.
      if lines_since_match > 2:
        # Merge second and subsequent message lines into the first line.
        remove_path_stacktrace_list[-1] += line
      else:
        remove_path_stacktrace_list.append(line)

  # Last line will be the exception type followed by message.
  # Remove the message since it could contain PII.
  exception_line = remove_path_stacktrace_list[-1]
  exception_line = encoding.Decode(exception_line).split(':', 1)[0]
  remove_path_stacktrace_list[-1] = exception_line

  formatted_stacktrace = '\n'.join(
      line for line in remove_path_stacktrace_list) + '\n'
  return formatted_stacktrace



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Pub/Sub topics publish command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.pubsub import topics
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.pubsub import flags
from googlecloudsdk.command_lib.pubsub import resource_args
from googlecloudsdk.command_lib.pubsub import util
from googlecloudsdk.core import properties
from googlecloudsdk.core.resource import resource_projector
from googlecloudsdk.core.util import http_encoding


def _Run(args, message_body, legacy_output=False):
  """Publishes a message to a topic."""
  client = topics.TopicsClient()

  attributes = util.ParseAttributes(args.attribute, messages=client.messages)
  ordering_key = getattr(args, 'ordering_key', None)
  topic_ref = args.CONCEPTS.topic.Parse()

  result = client.Publish(topic_ref, http_encoding.Encode(message_body),
                          attributes, ordering_key)

  if legacy_output:
    # We only allow to publish one message at a time, so do not return a
    # list of messageId.
    result = resource_projector.MakeSerializable(result)
    result['messageIds'] = result['messageIds'][0]
  return result


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Publish(base.Command):
  """Publishes a message to the specified topic."""

  detailed_help = {
      'DESCRIPTION':
          """\
          Publishes a message to the specified topic name for testing and
          troubleshooting. Use with caution: all associated subscribers must
          be able to consume and acknowledge any message you publish,
          otherwise the system will continuously re-attempt delivery of the
          bad message for 7 days.""",
      'EXAMPLES':
          """\
          To publish messages in a batch to a specific Cloud Pub/Sub topic,
          run:

            $ {command} mytopic --message="Hello World!" --attribute=KEY1=VAL1,KEY2=VAL2
      """
  }

  @classmethod
  def Args(cls, parser):
    resource_args.AddTopicResourceArg(parser, 'to publish messages to.')
    flags.AddPublishMessageFlags(parser)

  def Run(self, args):
    return _Run(args, args.message)


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class PublishBeta(Publish):
  """Publishes a message to the specified topic."""

  @classmethod
  def Args(cls, parser):
    resource_args.AddTopicResourceArg(parser, 'to publish messages to.')
    flags.AddPublishMessageFlags(parser, add_deprecated=True)

  def Run(self, args):
    message_body = flags.ParseMessageBody(args)
    legacy_output = properties.VALUES.pubsub.legacy_output.GetBool()
    return _Run(args, message_body, legacy_output=legacy_output)



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Pretty-print utilities.

Usage:

pretty_print.Success('Woo')
pretty_print.Info('doing {thing}', thing='something')  # works like .format()

✓ Woo  (the checkbox will be green)
  doing something

Bold and italic standard formatters are available (in conjunction with
reset), e.g:

pretty_print.Success('Let me {bold}stress{reset} the {italic}importance{reset}')

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_attr

READY_COLUMN = ('ready_symbol.color(red="[xX]",'
                'green="[\N{CHECK MARK}\N{HEAVY CHECK MARK}]",'
                'yellow="[-!\N{HORIZONTAL ELLIPSIS}]"):label=""')


def _Print(prefix, color, msg, **formatters):
  """Helper function to avoid import-time races."""
  con = console_attr.GetConsoleAttr()
  con.Colorize(prefix, color)
  formatters = formatters.copy()
  formatters.update({
      'bold': con.GetFontCode(bold=True),
      'italic': con.GetFontCode(italic=True),
      'reset': con.GetFontCode(),
  })
  log.status.Print(msg.format(**formatters))


def Success(msg, **formatters):
  """Print a nice little green checkbox, and a message.

  Args:
    msg: str, message accepting standard formatters.
    **formatters: extra args acting like .format()
  """
  _Print('✓ ', 'green', msg, **formatters)


def Info(msg, **formatters):
  """Simple print, with added left margin for alignment.

  Args:
    msg: str, message accepting standard formatters.
    **formatters: extra args acting like .format()
  """
  _Print('  ', None, msg, **formatters)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for AI Platform Tensorboard experiments API."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import common_args
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import errors
from googlecloudsdk.command_lib.util.args import labels_util


class TensorboardExperimentsClient(object):
  """High-level client for the AI Platform Tensorboard experiment surface."""

  def __init__(self,
               client=None,
               messages=None,
               version=constants.BETA_VERSION):
    self.client = client or apis.GetClientInstance(
        constants.AI_PLATFORM_API_NAME,
        constants.AI_PLATFORM_API_VERSION[version])
    self.messages = messages or self.client.MESSAGES_MODULE
    self._service = self.client.projects_locations_tensorboards_experiments
    self._version = version

  def Create(self, tensorboard_ref, args):
    if self._version == constants.ALPHA_VERSION:
      return self.CreateAlpha(tensorboard_ref, args)
    else:
      return self.CreateBeta(tensorboard_ref, args)

  def CreateBeta(self, tensorboard_ref, args):
    """Create a new Tensorboard experiment."""
    labels = labels_util.ParseCreateArgs(
        args, self.messages.GoogleCloudAiplatformV1beta1TensorboardExperiment
        .LabelsValue)
    request = self.messages.AiplatformProjectsLocationsTensorboardsExperimentsCreateRequest(
        parent=tensorboard_ref.RelativeName(),
        googleCloudAiplatformV1beta1TensorboardExperiment=self.messages
        .GoogleCloudAiplatformV1beta1TensorboardExperiment(
            displayName=args.display_name,
            description=args.description,
            labels=labels),
        tensorboardExperimentId=args.tensorboard_experiment_id)
    return self._service.Create(request)

  def CreateAlpha(self, tensorboard_ref, args):
    """Create a new Tensorboard experiment."""
    labels = labels_util.ParseCreateArgs(
        args, self.messages.GoogleCloudAiplatformV1alpha1TensorboardExperiment
        .LabelsValue)
    request = self.messages.AiplatformProjectsLocationsTensorboardsExperimentsCreateRequest(
        parent=tensorboard_ref.RelativeName(),
        googleCloudAiplatformV1alpha1TensorboardExperiment=self.messages
        .GoogleCloudAiplatformV1alpha1TensorboardExperiment(
            displayName=args.display_name,
            description=args.description,
            labels=labels),
        tensorboardExperimentId=args.tensorboard_experiment_id)
    return self._service.Create(request)

  def List(self, tensorboard_ref, limit=1000, page_size=50, sort_by=None):
    request = self.messages.AiplatformProjectsLocationsTensorboardsExperimentsListRequest(
        parent=tensorboard_ref.RelativeName(),
        orderBy=common_args.ParseSortByArg(sort_by))
    return list_pager.YieldFromList(
        self._service,
        request,
        field='tensorboardExperiments',
        batch_size_attribute='pageSize',
        batch_size=page_size,
        limit=limit)

  def Get(self, tensorboard_exp_ref):
    request = self.messages.AiplatformProjectsLocationsTensorboardsExperimentsGetRequest(
        name=tensorboard_exp_ref.RelativeName())
    return self._service.Get(request)

  def Delete(self, tensorboard_exp_ref):
    request = (
        self.messages
        .AiplatformProjectsLocationsTensorboardsExperimentsDeleteRequest(
            name=tensorboard_exp_ref.RelativeName()))
    return self._service.Delete(request)

  def Patch(self, tensorboard_exp_ref, args):
    if self._version == constants.ALPHA_VERSION:
      return self.PatchAlpha(tensorboard_exp_ref, args)
    else:
      return self.PatchBeta(tensorboard_exp_ref, args)

  def PatchBeta(self, tensorboard_exp_ref, args):
    """Update a Tensorboard experiment."""
    tensorboard_exp = (
        self.messages.GoogleCloudAiplatformV1beta1TensorboardExperiment())
    update_mask = []

    def GetLabels():
      return self.Get(tensorboard_exp_ref).labels

    labels_update = labels_util.ProcessUpdateArgsLazy(
        args, self.messages.GoogleCloudAiplatformV1beta1TensorboardExperiment
        .LabelsValue, GetLabels)
    if labels_update.needs_update:
      tensorboard_exp.labels = labels_update.labels
      update_mask.append('labels')

    if args.display_name is not None:
      tensorboard_exp.displayName = args.display_name
      update_mask.append('display_name')

    if args.description is not None:
      tensorboard_exp.description = args.description
      update_mask.append('description')

    if not update_mask:
      raise errors.NoFieldsSpecifiedError('No updates requested.')

    request = self.messages.AiplatformProjectsLocationsTensorboardsExperimentsPatchRequest(
        name=tensorboard_exp_ref.RelativeName(),
        googleCloudAiplatformV1beta1TensorboardExperiment=tensorboard_exp,
        updateMask=','.join(update_mask))
    return self._service.Patch(request)

  def PatchAlpha(self, tensorboard_exp_ref, args):
    """Update a Tensorboard experiment."""
    tensorboard_exp = (
        self.messages.GoogleCloudAiplatformV1alpha1TensorboardExperiment())
    update_mask = []

    def GetLabels():
      return self.Get(tensorboard_exp_ref).labels

    labels_update = labels_util.ProcessUpdateArgsLazy(
        args, self.messages.GoogleCloudAiplatformV1alpha1TensorboardExperiment
        .LabelsValue, GetLabels)
    if labels_update.needs_update:
      tensorboard_exp.labels = labels_update.labels
      update_mask.append('labels')

    if args.display_name is not None:
      tensorboard_exp.displayName = args.display_name
      update_mask.append('display_name')

    if args.description is not None:
      tensorboard_exp.description = args.description
      update_mask.append('description')

    if not update_mask:
      raise errors.NoFieldsSpecifiedError('No updates requested.')

    request = self.messages.AiplatformProjectsLocationsTensorboardsExperimentsPatchRequest(
        name=tensorboard_exp_ref.RelativeName(),
        googleCloudAiplatformV1alpha1TensorboardExperiment=tensorboard_exp,
        updateMask=','.join(update_mask))
    return self._service.Patch(request)



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Resource display taps.

A tapper is an object that converts an iterable into another iterable. It
applies the Tap method as each item is iterated on, and applies the Done method
just before the iteration stops. A Tapper may delete items from an iterable
and/or inject new items into an iterable. Tappers are useful for monitoring or
modifying an iterable without consuming it all at once.

For example one could always:

  items = list(iterable_items)
  for item in items:
    do_something(item)
  # pass items to the next consumer

However, if an iterable consumed a lot of resources (say a generator with a
zillion items) then the list() statement would instantiate all of the items and
strain memory. A tapper avoids this by dealing with each item as it is
generated.

There is a tapper for each of the --filter, --flatten, --limit, and --page
flags.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import log
from googlecloudsdk.core.resource import resource_filter
from googlecloudsdk.core.resource import resource_printer_base
from googlecloudsdk.core.resource import resource_projector
from googlecloudsdk.core.resource import resource_property
from googlecloudsdk.core.util import peek_iterable


class Filterer(peek_iterable.Tap):
  """A Tapper class that filters out resources not matching an expression.

  Attributes:
    _match: The resource filter method.
  """

  def __init__(self, expression, defaults):
    """Constructor.

    Args:
      expression: The resource filter expression string.
      defaults: The resource format and filter default projection.
    """
    self._compiled_expression = resource_filter.Compile(
        expression, defaults=defaults)
    self._missing_keys = resource_filter.GetAllKeys(self._compiled_expression)

  def Tap(self, resource):
    """Returns True if resource matches the filter expression.

    Args:
      resource: The resource to filter.

    Returns:
      True if resource matches the filter expression.
    """
    self._missing_keys -= set(
        key for key in self._missing_keys
        if resource_property.ResourceContainsKey(
            resource_projector.MakeSerializable(resource), key))
    if resource_printer_base.IsResourceMarker(resource):
      return True
    return self._compiled_expression.Evaluate(
        resource_projector.MakeSerializable(resource))

  def Done(self):

    def WarnMissingKeys(missing_keys):
      missing_keys_str = ', '.join(
          ['.'.join(map(str, key)) for key in sorted(missing_keys)])
      log.warning(
          'The following filter keys were not present in any resource : ' +
          missing_keys_str)

    if self._missing_keys:
      WarnMissingKeys(self._missing_keys)


class Flattener(peek_iterable.Tap):
  """A Tapper class that flattens a resource key slice to separate records.

  A serialized copy of the resource is modified in place. This means the same
  resource object is returned for each flattened slice item. This is OK because
  the downstream is not guaranteed uniqueness.

  Attributes:
    _child_name: The flattened value to set is _parent_key[_child_name].
    _key: The parsed resource key of the slice to flatten.
    _parent_key: The parent of _key, None for the resource itself.
    _items: The items to flatten in the current resource.
    _resource: The serialized copy of the current resource.
  """

  def __init__(self, key):
    """Constructor.

    Args:
      key: The resource key of the slice to flatten.
    """
    # Save a copy of the key.
    self._key = key[:]
    self._child_name = self._key[-1] if self._key else None
    self._parent_key = self._key[:-1] if self._key else None
    self._items = None
    self._resource = None

  def Tap(self, resource):
    """Returns the next slice item in resource.

    Args:
      resource: The resource to flatten.

    Returns:
      True if the next slice is not a list, False if there are no more items,
      or Injector(resource) which is the resource with the next slice flattened.
    """
    if self._items is None:
      # Modify a serialized copy of resource.
      self._resource = resource_projector.MakeSerializable(resource)
      self._items = resource_property.Get(self._resource, self._key)
      if not isinstance(self._items, list):
        item = self._items
        self._items = None
        return peek_iterable.TapInjector(item, replace=True)
    if not self._items:
      self._items = None
      return False
    item = self._items.pop(0)
    if self._parent_key:
      parent = resource_property.Get(self._resource, self._parent_key)
    else:
      parent = self._resource
    parent[self._child_name] = item
    return peek_iterable.TapInjector(
        resource_projector.MakeSerializable(self._resource))


class Limiter(peek_iterable.Tap):
  """A Tapper class that filters out resources after a limit is reached.

  Attributes:
    _limit: The resource count limit.
    _count: The resource count.
  """

  def __init__(self, limit):
    self._limit = limit
    self._count = 0

  def Tap(self, resource):
    """Returns True if the limit has not been reached yet, None otherwise.

    Args:
      resource: The resource to limit.

    Returns:
      True if the limit has not been reached yet, None otherwise to stop
      iterations.
    """
    if resource_printer_base.IsResourceMarker(resource):
      return True
    self._count += 1
    return self._count <= self._limit or None


class Pager(peek_iterable.Tap):
  """A Tapper class that injects a PageMarker after each page of resources.

  Attributes:
    _page_size: The number of resources per page.
    _count: The current page resource count.
  """

  def __init__(self, page_size):
    self._page_size = page_size
    self._count = 0

  def Tap(self, resource):
    """Injects a PageMarker if the current page limit has been reached.

    Args:
      resource: The resource to limit.

    Returns:
      TapInjector(PageMarker) if the page current page limit has been reached,
      otherwise True to retain the current resource.
    """
    if resource_printer_base.IsResourceMarker(resource):
      return True
    self._count += 1
    if self._count > self._page_size:
      self._count = 0
      return peek_iterable.TapInjector(resource_printer_base.PageMarker())
    return True


class UriCacher(peek_iterable.Tap):
  """A Tapper class that caches URIs based on the cache update op.

  Attributes:
    _transform_uri: The uri() transform function.
    _update_cache_op: The non-None return value from UpdateUriCache().
    _uris: The list of changed URIs, None if it is corrupt.
  """

  def __init__(self, update_cache_op, transform_uri):
    self._transform_uri = transform_uri
    self._update_cache_op = update_cache_op
    self._uris = []

  def Tap(self, resource):
    """Appends the URI for resource to the list of cache changes.

    Sets self._uris to None if a URI could not be retrieved for any resource.

    Args:
      resource: The resource from which the URI is extracted.

    Returns:
      True - all resources are seen downstream.
    """
    if resource_printer_base.IsResourceMarker(resource):
      return True
    if self._uris is not None:
      uri = self._transform_uri(resource, undefined=None)
      if uri:
        self._uris.append(uri)
      else:
        self._uris = None
    return True

  def Done(self):
    if self._uris is not None:
      self._update_cache_op.Update(self._uris)


class UriReplacer(peek_iterable.Tap):
  """A Tapper class that replaces each resource item with its URI.

  Attributes:
    _transform_uri: The uri() transform function.
  """

  def __init__(self, transform_uri):
    self._transform_uri = transform_uri

  def Tap(self, resource):
    """Replaces resource with its URI or skips the resource if it has no URI.

    Args:
      resource: The resource to replace with its URI.

    Returns:
      TapInjector(URI, replace=True) if the resource has a URI or False to skip
      the resource.
    """
    if resource_printer_base.IsResourceMarker(resource):
      return True
    uri = self._transform_uri(resource, undefined=None)
    if not uri:
      return False
    return peek_iterable.TapInjector(uri, replace=True)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Create hooks for Cloud Game Servers Config."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.game.servers import utils


def ProcessConfigsFiles(ref, args, request):
  """Reads the fleet/scaling configs into FleetConfig/ScalingConfig protos and updates the request."""
  del ref
  if args.fleet_configs_file:
    request.gameServerConfig.fleetConfigs = utils.ProcessFleetConfigsFile(
        args.fleet_configs_file, utils.GetApiVersionFromArgs(args))

  if args.scaling_configs_file:
    request.gameServerConfig.scalingConfigs = utils.ProcessScalingConfigsFile(
        args.scaling_configs_file, utils.GetApiVersionFromArgs(args))

  return request



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Support library to handle the staging bucket."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import properties


# This function is very similar to the one made for Cloud Build.
def GetDefaultStagingBucket():
  """Returns the default bucket stage files.

  Returns:
    A string representing the GCS bucket name.
  """
  safe_project = (properties.VALUES.core.project.Get(required=True)
                  .replace(':', '_')
                  .replace('.', '_')
                  # The string 'google' is not allowed in bucket names.
                  .replace('google', 'elgoog'))

  return safe_project + '_blueprints'



"""Generated client library for workflowexecutions version v1alpha1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.third_party.apis.workflowexecutions.v1alpha1 import workflowexecutions_v1alpha1_messages as messages


class WorkflowexecutionsV1alpha1(base_api.BaseApiClient):
  """Generated client library for service workflowexecutions version v1alpha1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://workflowexecutions.googleapis.com/'
  MTLS_BASE_URL = 'https://workflowexecutions.mtls.googleapis.com/'

  _PACKAGE = 'workflowexecutions'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'WorkflowexecutionsV1alpha1'
  _URL_VERSION = 'v1alpha1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new workflowexecutions handle."""
    url = url or self.BASE_URL
    super(WorkflowexecutionsV1alpha1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_workflows_executions = self.ProjectsLocationsWorkflowsExecutionsService(self)
    self.projects_locations_workflows = self.ProjectsLocationsWorkflowsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsWorkflowsExecutionsService(base_api.BaseApiService):
    """Service class for the projects_locations_workflows_executions resource."""

    _NAME = 'projects_locations_workflows_executions'

    def __init__(self, client):
      super(WorkflowexecutionsV1alpha1.ProjectsLocationsWorkflowsExecutionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Cancels a workflow execution of the given name.

      Args:
        request: (WorkflowexecutionsProjectsLocationsWorkflowsExecutionsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Execution) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/workflows/{workflowsId}/executions/{executionsId}:cancel',
        http_method='POST',
        method_id='workflowexecutions.projects.locations.workflows.executions.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}:cancel',
        request_field='cancelExecutionRequest',
        request_type_name='WorkflowexecutionsProjectsLocationsWorkflowsExecutionsCancelRequest',
        response_type_name='Execution',
        supports_download=False,
    )

    def Create(self, request, global_params=None):
      r"""Creates a new workflow execution using the latest version of the workflow.

      Args:
        request: (WorkflowexecutionsProjectsLocationsWorkflowsExecutionsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Execution) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/workflows/{workflowsId}/executions',
        http_method='POST',
        method_id='workflowexecutions.projects.locations.workflows.executions.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1alpha1/{+parent}/executions',
        request_field='execution',
        request_type_name='WorkflowexecutionsProjectsLocationsWorkflowsExecutionsCreateRequest',
        response_type_name='Execution',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Returns a workflow execution with the specified name.

      Args:
        request: (WorkflowexecutionsProjectsLocationsWorkflowsExecutionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Execution) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/workflows/{workflowsId}/executions/{executionsId}',
        http_method='GET',
        method_id='workflowexecutions.projects.locations.workflows.executions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha1/{+name}',
        request_field='',
        request_type_name='WorkflowexecutionsProjectsLocationsWorkflowsExecutionsGetRequest',
        response_type_name='Execution',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Returns a list of workflow executions which belong to the workflow with the specified name. The method returns executions from all workflow versions.

      Args:
        request: (WorkflowexecutionsProjectsLocationsWorkflowsExecutionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListExecutionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha1/projects/{projectsId}/locations/{locationsId}/workflows/{workflowsId}/executions',
        http_method='GET',
        method_id='workflowexecutions.projects.locations.workflows.executions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha1/{+parent}/executions',
        request_field='',
        request_type_name='WorkflowexecutionsProjectsLocationsWorkflowsExecutionsListRequest',
        response_type_name='ListExecutionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsWorkflowsService(base_api.BaseApiService):
    """Service class for the projects_locations_workflows resource."""

    _NAME = 'projects_locations_workflows'

    def __init__(self, client):
      super(WorkflowexecutionsV1alpha1.ProjectsLocationsWorkflowsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(WorkflowexecutionsV1alpha1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(WorkflowexecutionsV1alpha1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utility functions for gcloud datastore emulator."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import os
import tempfile
from googlecloudsdk.command_lib.emulators import util
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import execution_utils
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.util import platforms


class UnableToPrepareDataDir(exceptions.Error):

  def __init__(self):
    super(UnableToPrepareDataDir, self).__init__(
        'Unable to prepare the data directory for the emulator')


def ArgsForGCDEmulator(emulator_args):
  """Constructs an argument list for calling the GCD emulator.

  Args:
    emulator_args: args for the emulator.

  Returns:
    An argument list to execute the GCD emulator.
  """
  current_os = platforms.OperatingSystem.Current()
  if current_os is platforms.OperatingSystem.WINDOWS:
    cmd = 'cloud_datastore_emulator.cmd'
    gcd_executable = os.path.join(util.GetEmulatorRoot(CLOUD_DATASTORE), cmd)
    return execution_utils.ArgsForCMDTool(gcd_executable, *emulator_args)
  else:
    cmd = 'cloud_datastore_emulator'
    gcd_executable = os.path.join(util.GetEmulatorRoot(CLOUD_DATASTORE), cmd)
    return execution_utils.ArgsForExecutableTool(gcd_executable, *emulator_args)


DATASTORE = 'datastore'
CLOUD_DATASTORE = 'cloud-datastore'
DATASTORE_TITLE = 'Google Cloud Datastore emulator'


def PrepareGCDDataDir(args):
  """Prepares the given directory using gcd create.

  Raises:
    UnableToPrepareDataDir: If the gcd create execution fails.

  Args:
    args: The arguments passed to the command.
  """
  data_dir = args.data_dir
  if os.path.isdir(data_dir) and os.listdir(data_dir):
    log.warning('Reusing existing data in [{0}].'.format(data_dir))
    return

  gcd_create_args = ['create']
  project = properties.VALUES.core.project.Get(required=True)
  gcd_create_args.append('--project_id={0}'.format(project))
  gcd_create_args.append(data_dir)
  exec_args = ArgsForGCDEmulator(gcd_create_args)

  log.status.Print('Executing: {0}'.format(' '.join(exec_args)))
  with util.Exec(exec_args) as process:
    util.PrefixOutput(process, DATASTORE)
    failed = process.poll()
    if failed:
      raise UnableToPrepareDataDir()


def StartGCDEmulator(args, log_file=None):
  """Starts the datastore emulator with the given arguments.

  Args:
    args: Arguments passed to the start command.
    log_file: optional file argument to reroute process's output.

  Returns:
    process, The handle of the child process running the datastore emulator.
  """
  gcd_start_args = ['start']
  gcd_start_args.append('--host={0}'.format(args.host_port.host))
  gcd_start_args.append('--port={0}'.format(args.host_port.port))
  gcd_start_args.append('--store_on_disk={0}'.format(args.store_on_disk))
  gcd_start_args.append('--allow_remote_shutdown')
  if args.use_firestore_in_datastore_mode:
    gcd_start_args.append('--firestore_in_datastore_mode')
  else:
    gcd_start_args.append('--consistency={0}'.format(args.consistency))
  gcd_start_args.append(args.data_dir)
  exec_args = ArgsForGCDEmulator(gcd_start_args)

  log.status.Print('Executing: {0}'.format(' '.join(exec_args)))
  return util.Exec(exec_args, log_file=log_file)


def WriteGCDEnvYaml(args):
  """Writes the env.yaml file for the datastore emulator with provided args.

  Args:
    args: Arguments passed to the start command.
  """
  host_port = '{0}:{1}'.format(args.host_port.host, args.host_port.port)
  project_id = properties.VALUES.core.project.Get(required=True)
  env = {'DATASTORE_HOST': 'http://{0}'.format(host_port),
         'DATASTORE_EMULATOR_HOST': host_port,
         'DATASTORE_EMULATOR_HOST_PATH': '{0}/datastore'.format(host_port),
         'DATASTORE_DATASET': project_id,
         'DATASTORE_PROJECT_ID': project_id,
        }
  util.WriteEnvYaml(env, args.data_dir)


def GetDataDir():
  return util.GetDataDir(DATASTORE)


def GetHostPort():
  return util.GetHostPort(DATASTORE)


class DatastoreEmulator(util.Emulator):
  """Represents the ability to start and route datastore emulator."""

  def Start(self, port):
    args = util.AttrDict({
        'host_port': {
            'host': 'localhost',
            'port': port,
        },
        'store_on_disk': True,
        'consistency': 0.9,
        'data_dir': tempfile.mkdtemp(),
    })
    PrepareGCDDataDir(args)
    return StartGCDEmulator(args, self._GetLogNo())

  @property
  def prefixes(self):
    # Taken Jan 1, 2017 from:
    # https://cloud.google.com/datastore/docs/reference/rpc/google.datastore.v1
    # Note that this should probably be updated to just be based off of the
    # prefix, without enumerating all of the types.
    return [
        'google.datastore.v1.Datastore',
        'google.datastore.v1.AllocateIdsRequest',
        'google.datastore.v1.AllocateIdsResponse',
        'google.datastore.v1.ArrayValue',
        'google.datastore.v1.BeginTransactionRequest',
        'google.datastore.v1.BeginTransactionResponse',
        'google.datastore.v1.CommitRequest',
        'google.datastore.v1.CommitRequest.Mode',
        'google.datastore.v1.CommitResponse',
        'google.datastore.v1.CompositeFilter',
        'google.datastore.v1.CompositeFilter.Operator',
        'google.datastore.v1.Entity',
        'google.datastore.v1.EntityResult',
        'google.datastore.v1.EntityResult.ResultType',
        'google.datastore.v1.Filter',
        'google.datastore.v1.GqlQuery',
        'google.datastore.v1.GqlQueryParameter',
        'google.datastore.v1.Key',
        'google.datastore.v1.Key.PathElement',
        'google.datastore.v1.KindExpression',
        'google.datastore.v1.LookupRequest',
        'google.datastore.v1.LookupResponse',
        'google.datastore.v1.Mutation',
        'google.datastore.v1.MutationResult',
        'google.datastore.v1.PartitionId',
        'google.datastore.v1.Projection',
        'google.datastore.v1.PropertyFilter',
        'google.datastore.v1.PropertyFilter.Operator',
        'google.datastore.v1.PropertyOrder',
        'google.datastore.v1.PropertyOrder.Direction',
        'google.datastore.v1.PropertyReference',
        'google.datastore.v1.Query',
        'google.datastore.v1.QueryResultBatch',
        'google.datastore.v1.QueryResultBatch.MoreResultsType',
        'google.datastore.v1.ReadOptions',
        'google.datastore.v1.ReadOptions.ReadConsistency'
        'google.datastore.v1.RollbackRequest',
        'google.datastore.v1.RollbackResponse',
        'google.datastore.v1.RunQueryRequest',
        'google.datastore.v1.RunQueryResponse',
        'google.datastore.v1.Value',
    ]

  @property
  def service_name(self):
    return DATASTORE

  @property
  def emulator_title(self):
    return DATASTORE_TITLE

  @property
  def emulator_component(self):
    return 'cloud-datastore-emulator'



"""Generated message classes for speech version v1p1beta1.

Converts audio to text by applying powerful neural network models.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'speech'


class ClassItem(_messages.Message):
  r"""An item of the class.

  Fields:
    value: The class item's value.
  """

  value = _messages.StringField(1)


class CreateCustomClassRequest(_messages.Message):
  r"""Message sent by the client for the `CreateCustomClass` method.

  Fields:
    customClass: Required. The custom class to create.
    customClassId: Required. The ID to use for the custom class, which will
      become the final component of the custom class' resource name. This
      value should restrict to letters, numbers, and hyphens, with the first
      character a letter, the last a letter or a number, and be 4-63
      characters.
  """

  customClass = _messages.MessageField('CustomClass', 1)
  customClassId = _messages.StringField(2)


class CreatePhraseSetRequest(_messages.Message):
  r"""Message sent by the client for the `CreatePhraseSet` method.

  Fields:
    phraseSet: Required. The phrase set to create.
    phraseSetId: Required. The ID to use for the phrase set, which will become
      the final component of the phrase set's resource name. This value should
      restrict to letters, numbers, and hyphens, with the first character a
      letter, the last a letter or a number, and be 4-63 characters.
  """

  phraseSet = _messages.MessageField('PhraseSet', 1)
  phraseSetId = _messages.StringField(2)


class CustomClass(_messages.Message):
  r"""A set of words or phrases that represents a common concept likely to
  appear in your audio, for example a list of passenger ship names.
  CustomClass items can be substituted into placeholders that you set in
  PhraseSet phrases.

  Fields:
    customClassId: If this custom class is a resource, the custom_class_id is
      the resource id of the CustomClass. Case sensitive.
    items: A collection of class items.
    name: The resource name of the custom class.
  """

  customClassId = _messages.StringField(1)
  items = _messages.MessageField('ClassItem', 2, repeated=True)
  name = _messages.StringField(3)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class Entry(_messages.Message):
  r"""A single replacement configuration.

  Fields:
    caseSensitive: Whether the search is case sensitive.
    replace: What to replace with. Max length is 100 characters.
    search: What to replace. Max length is 100 characters.
  """

  caseSensitive = _messages.BooleanField(1)
  replace = _messages.StringField(2)
  search = _messages.StringField(3)


class ListCustomClassesResponse(_messages.Message):
  r"""Message returned to the client by the `ListCustomClasses` method.

  Fields:
    customClasses: The custom classes.
    nextPageToken: A token, which can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
  """

  customClasses = _messages.MessageField('CustomClass', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)


class ListPhraseSetResponse(_messages.Message):
  r"""Message returned to the client by the `ListPhraseSet` method.

  Fields:
    nextPageToken: A token, which can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
    phraseSets: The phrase set.
  """

  nextPageToken = _messages.StringField(1)
  phraseSets = _messages.MessageField('PhraseSet', 2, repeated=True)


class LongRunningRecognizeMetadata(_messages.Message):
  r"""Describes the progress of a long-running `LongRunningRecognize` call. It
  is included in the `metadata` field of the `Operation` returned by the
  `GetOperation` call of the `google::longrunning::Operations` service.

  Fields:
    lastUpdateTime: Time of the most recent processing update.
    outputConfig: Output only. A copy of the TranscriptOutputConfig if it was
      set in the request.
    progressPercent: Approximate percentage of audio processed thus far.
      Guaranteed to be 100 when the audio is fully processed and the results
      are available.
    startTime: Time when the request was received.
    uri: Output only. The URI of the audio file being transcribed. Empty if
      the audio was sent as byte content.
  """

  lastUpdateTime = _messages.StringField(1)
  outputConfig = _messages.MessageField('TranscriptOutputConfig', 2)
  progressPercent = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  startTime = _messages.StringField(4)
  uri = _messages.StringField(5)


class LongRunningRecognizeRequest(_messages.Message):
  r"""The top-level message sent by the client for the `LongRunningRecognize`
  method.

  Fields:
    audio: Required. The audio data to be recognized.
    config: Required. Provides information to the recognizer that specifies
      how to process the request.
    outputConfig: Optional. Specifies an optional destination for the
      recognition results.
  """

  audio = _messages.MessageField('RecognitionAudio', 1)
  config = _messages.MessageField('RecognitionConfig', 2)
  outputConfig = _messages.MessageField('TranscriptOutputConfig', 3)


class LongRunningRecognizeResponse(_messages.Message):
  r"""The only message returned to the client by the `LongRunningRecognize`
  method. It contains the result as zero or more sequential
  `SpeechRecognitionResult` messages. It is included in the `result.response`
  field of the `Operation` returned by the `GetOperation` call of the
  `google::longrunning::Operations` service.

  Fields:
    outputConfig: Original output config if present in the request.
    outputError: If the transcript output fails this field contains the
      relevant error.
    requestId: The ID associated with the request. This is a unique ID
      specific only to the given request.
    results: Sequential list of transcription results corresponding to
      sequential portions of audio.
    totalBilledTime: When available, billed audio seconds for the
      corresponding request.
  """

  outputConfig = _messages.MessageField('TranscriptOutputConfig', 1)
  outputError = _messages.MessageField('Status', 2)
  requestId = _messages.IntegerField(3)
  results = _messages.MessageField('SpeechRecognitionResult', 4, repeated=True)
  totalBilledTime = _messages.StringField(5)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal response of the operation in case of success. If
      the original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal response of the operation in case of success. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal response of the operation in case of success. If the
    original method returns no data on success, such as `Delete`, the response
    is `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class Phrase(_messages.Message):
  r"""A phrases containing words and phrase "hints" so that the speech
  recognition is more likely to recognize them. This can be used to improve
  the accuracy for specific words and phrases, for example, if specific
  commands are typically spoken by the user. This can also be used to add
  additional words to the vocabulary of the recognizer. See [usage
  limits](https://cloud.google.com/speech-to-text/quotas#content). List items
  can also include pre-built or custom classes containing groups of words that
  represent common concepts that occur in natural language. For example,
  rather than providing a phrase hint for every month of the year (e.g. "i was
  born in january", "i was born in febuary", ...), use the pre-built `$MONTH`
  class improves the likelihood of correctly transcribing audio that includes
  months (e.g. "i was born in $month"). To refer to pre-built classes, use the
  class' symbol prepended with `$` e.g. `$MONTH`. To refer to custom classes
  that were defined inline in the request, set the class's `custom_class_id`
  to a string unique to all class resources and inline classes. Then use the
  class' id wrapped in $`{...}` e.g. "${my-months}". To refer to custom
  classes resources, use the class' id wrapped in `${}` (e.g. `${my-months}`).
  Speech-to-Text supports three locations: `global`, `us` (US North America),
  and `eu` (Europe). If you are calling the `speech.googleapis.com` endpoint,
  use the `global` location. To specify a region, use a [regional
  endpoint](/speech-to-text/docs/endpoints) with matching `us` or `eu`
  location value.

  Fields:
    boost: Hint Boost. Overrides the boost set at the phrase set level.
      Positive value will increase the probability that a specific phrase will
      be recognized over other similar sounding phrases. The higher the boost,
      the higher the chance of false positive recognition as well. Negative
      boost will simply be ignored. Though `boost` can accept a wide range of
      positive values, most use cases are best served with values between 0
      and 20. We recommend using a binary search approach to finding the
      optimal value for your use case. Speech recognition will skip PhraseSets
      with a boost value of 0.
    value: The phrase itself.
  """

  boost = _messages.FloatField(1, variant=_messages.Variant.FLOAT)
  value = _messages.StringField(2)


class PhraseSet(_messages.Message):
  r"""Provides "hints" to the speech recognizer to favor specific words and
  phrases in the results.

  Fields:
    boost: Hint Boost. Positive value will increase the probability that a
      specific phrase will be recognized over other similar sounding phrases.
      The higher the boost, the higher the chance of false positive
      recognition as well. Negative boost values would correspond to anti-
      biasing. Anti-biasing is not enabled, so negative boost will simply be
      ignored. Though `boost` can accept a wide range of positive values, most
      use cases are best served with values between 0 (exclusive) and 20. We
      recommend using a binary search approach to finding the optimal value
      for your use case. Speech recognition will skip PhraseSets with a boost
      value of 0.
    name: The resource name of the phrase set.
    phrases: A list of word and phrases.
  """

  boost = _messages.FloatField(1, variant=_messages.Variant.FLOAT)
  name = _messages.StringField(2)
  phrases = _messages.MessageField('Phrase', 3, repeated=True)


class RecognitionAudio(_messages.Message):
  r"""Contains audio data in the encoding specified in the
  `RecognitionConfig`. Either `content` or `uri` must be supplied. Supplying
  both or neither returns google.rpc.Code.INVALID_ARGUMENT. See [content
  limits](https://cloud.google.com/speech-to-text/quotas#content).

  Fields:
    content: The audio data bytes encoded as specified in `RecognitionConfig`.
      Note: as with all bytes fields, proto buffers use a pure binary
      representation, whereas JSON representations use base64.
    uri: URI that points to a file that contains audio data bytes as specified
      in `RecognitionConfig`. The file must not be compressed (for example,
      gzip). Currently, only Google Cloud Storage URIs are supported, which
      must be specified in the following format:
      `gs://bucket_name/object_name` (other URI formats return
      google.rpc.Code.INVALID_ARGUMENT). For more information, see [Request
      URIs](https://cloud.google.com/storage/docs/reference-uris).
  """

  content = _messages.BytesField(1)
  uri = _messages.StringField(2)


class RecognitionConfig(_messages.Message):
  r"""Provides information to the recognizer that specifies how to process the
  request.

  Enums:
    EncodingValueValuesEnum: Encoding of audio data sent in all
      `RecognitionAudio` messages. This field is optional for `FLAC` and `WAV`
      audio files and required for all other audio formats. For details, see
      AudioEncoding.

  Fields:
    adaptation: Speech adaptation configuration improves the accuracy of
      speech recognition. For more information, see the [speech
      adaptation](https://cloud.google.com/speech-to-text/docs/adaptation)
      documentation. When speech adaptation is set it supersedes the
      `speech_contexts` field.
    alternativeLanguageCodes: A list of up to 3 additional
      [BCP-47](https://www.rfc-editor.org/rfc/bcp/bcp47.txt) language tags,
      listing possible alternative languages of the supplied audio. See
      [Language Support](https://cloud.google.com/speech-to-
      text/docs/languages) for a list of the currently supported language
      codes. If alternative languages are listed, recognition result will
      contain recognition in the most likely language detected including the
      main language_code. The recognition result will include the language tag
      of the language detected in the audio. Note: This feature is only
      supported for Voice Command and Voice Search use cases and performance
      may vary for other use cases (e.g., phone call transcription).
    audioChannelCount: The number of channels in the input audio data. ONLY
      set this for MULTI-CHANNEL recognition. Valid values for LINEAR16 and
      FLAC are `1`-`8`. Valid values for OGG_OPUS are '1'-'254'. Valid value
      for MULAW, AMR, AMR_WB and SPEEX_WITH_HEADER_BYTE is only `1`. If `0` or
      omitted, defaults to one channel (mono). Note: We only recognize the
      first channel by default. To perform independent recognition on each
      channel set `enable_separate_recognition_per_channel` to 'true'.
    diarizationConfig: Config to enable speaker diarization and set additional
      parameters to make diarization better suited for your application. Note:
      When this is enabled, we send all the words from the beginning of the
      audio for the top alternative in every consecutive STREAMING responses.
      This is done in order to improve our speaker tags as our models learn to
      identify the speakers in the conversation over time. For non-streaming
      requests, the diarization results will be provided only in the top
      alternative of the FINAL SpeechRecognitionResult.
    diarizationSpeakerCount: If set, specifies the estimated number of
      speakers in the conversation. Defaults to '2'. Ignored unless
      enable_speaker_diarization is set to true. Note: Use diarization_config
      instead.
    enableAutomaticPunctuation: If 'true', adds punctuation to recognition
      result hypotheses. This feature is only available in select languages.
      Setting this for requests in other languages has no effect at all. The
      default 'false' value does not add punctuation to result hypotheses.
    enableSeparateRecognitionPerChannel: This needs to be set to `true`
      explicitly and `audio_channel_count` > 1 to get each channel recognized
      separately. The recognition result will contain a `channel_tag` field to
      state which channel that result belongs to. If this is not true, we will
      only recognize the first channel. The request is billed cumulatively for
      all channels recognized: `audio_channel_count` multiplied by the length
      of the audio.
    enableSpeakerDiarization: If 'true', enables speaker detection for each
      recognized word in the top alternative of the recognition result using a
      speaker_tag provided in the WordInfo. Note: Use diarization_config
      instead.
    enableSpokenEmojis: The spoken emoji behavior for the call If not set,
      uses default behavior based on model of choice If 'true', adds spoken
      emoji formatting for the request. This will replace spoken emojis with
      the corresponding Unicode symbols in the final transcript. If 'false',
      spoken emojis are not replaced.
    enableSpokenPunctuation: The spoken punctuation behavior for the call If
      not set, uses default behavior based on model of choice e.g.
      command_and_search will enable spoken punctuation by default If 'true',
      replaces spoken punctuation with the corresponding symbols in the
      request. For example, "how are you question mark" becomes "how are
      you?". See https://cloud.google.com/speech-to-text/docs/spoken-
      punctuation for support. If 'false', spoken punctuation is not replaced.
    enableWordConfidence: If `true`, the top result includes a list of words
      and the confidence for those words. If `false`, no word-level confidence
      information is returned. The default is `false`.
    enableWordTimeOffsets: If `true`, the top result includes a list of words
      and the start and end time offsets (timestamps) for those words. If
      `false`, no word-level time offset information is returned. The default
      is `false`.
    encoding: Encoding of audio data sent in all `RecognitionAudio` messages.
      This field is optional for `FLAC` and `WAV` audio files and required for
      all other audio formats. For details, see AudioEncoding.
    languageCode: Required. The language of the supplied audio as a
      [BCP-47](https://www.rfc-editor.org/rfc/bcp/bcp47.txt) language tag.
      Example: "en-US". See [Language
      Support](https://cloud.google.com/speech-to-text/docs/languages) for a
      list of the currently supported language codes.
    maxAlternatives: Maximum number of recognition hypotheses to be returned.
      Specifically, the maximum number of `SpeechRecognitionAlternative`
      messages within each `SpeechRecognitionResult`. The server may return
      fewer than `max_alternatives`. Valid values are `0`-`30`. A value of `0`
      or `1` will return a maximum of one. If omitted, will return a maximum
      of one.
    metadata: Metadata regarding this request.
    model: Which model to select for the given request. Select the model best
      suited to your domain to get best results. If a model is not explicitly
      specified, then we auto-select a model based on the parameters in the
      RecognitionConfig. *Model* *Description* latest_long Best for long form
      content like media or conversation. latest_short Best for short form
      content like commands or single shot directed speech. command_and_search
      Best for short queries such as voice commands or voice search.
      phone_call Best for audio that originated from a phone call (typically
      recorded at an 8khz sampling rate). video Best for audio that originated
      from video or includes multiple speakers. Ideally the audio is recorded
      at a 16khz or greater sampling rate. This is a premium model that costs
      more than the standard rate. default Best for audio that is not one of
      the specific audio models. For example, long-form audio. Ideally the
      audio is high-fidelity, recorded at a 16khz or greater sampling rate.
      medical_conversation Best for audio that originated from a conversation
      between a medical provider and patient. medical_dictation Best for audio
      that originated from dictation notes by a medical provider.
    profanityFilter: If set to `true`, the server will attempt to filter out
      profanities, replacing all but the initial character in each filtered
      word with asterisks, e.g. "f***". If set to `false` or omitted,
      profanities won't be filtered out.
    sampleRateHertz: Sample rate in Hertz of the audio data sent in all
      `RecognitionAudio` messages. Valid values are: 8000-48000. 16000 is
      optimal. For best results, set the sampling rate of the audio source to
      16000 Hz. If that's not possible, use the native sample rate of the
      audio source (instead of re-sampling). This field is optional for FLAC
      and WAV audio files, but is required for all other audio formats. For
      details, see AudioEncoding.
    speechContexts: Array of SpeechContext. A means to provide context to
      assist the speech recognition. For more information, see [speech
      adaptation](https://cloud.google.com/speech-to-text/docs/adaptation).
    transcriptNormalization: Use transcription normalization to automatically
      replace parts of the transcript with phrases of your choosing. For
      StreamingRecognize, this normalization only applies to stable partial
      transcripts (stability > 0.8) and final transcripts.
    useEnhanced: Set to true to use an enhanced model for speech recognition.
      If `use_enhanced` is set to true and the `model` field is not set, then
      an appropriate enhanced model is chosen if an enhanced model exists for
      the audio. If `use_enhanced` is true and an enhanced version of the
      specified model does not exist, then the speech is recognized using the
      standard version of the specified model.
  """

  class EncodingValueValuesEnum(_messages.Enum):
    r"""Encoding of audio data sent in all `RecognitionAudio` messages. This
    field is optional for `FLAC` and `WAV` audio files and required for all
    other audio formats. For details, see AudioEncoding.

    Values:
      ENCODING_UNSPECIFIED: Not specified.
      LINEAR16: Uncompressed 16-bit signed little-endian samples (Linear PCM).
      FLAC: `FLAC` (Free Lossless Audio Codec) is the recommended encoding
        because it is lossless--therefore recognition is not compromised--and
        requires only about half the bandwidth of `LINEAR16`. `FLAC` stream
        encoding supports 16-bit and 24-bit samples, however, not all fields
        in `STREAMINFO` are supported.
      MULAW: 8-bit samples that compand 14-bit audio samples using G.711
        PCMU/mu-law.
      AMR: Adaptive Multi-Rate Narrowband codec. `sample_rate_hertz` must be
        8000.
      AMR_WB: Adaptive Multi-Rate Wideband codec. `sample_rate_hertz` must be
        16000.
      OGG_OPUS: Opus encoded audio frames in Ogg container
        ([OggOpus](https://wiki.xiph.org/OggOpus)). `sample_rate_hertz` must
        be one of 8000, 12000, 16000, 24000, or 48000.
      SPEEX_WITH_HEADER_BYTE: Although the use of lossy encodings is not
        recommended, if a very low bitrate encoding is required, `OGG_OPUS` is
        highly preferred over Speex encoding. The [Speex](https://speex.org/)
        encoding supported by Cloud Speech API has a header byte in each
        block, as in MIME type `audio/x-speex-with-header-byte`. It is a
        variant of the RTP Speex encoding defined in [RFC
        5574](https://tools.ietf.org/html/rfc5574). The stream is a sequence
        of blocks, one block per RTP packet. Each block starts with a byte
        containing the length of the block, in bytes, followed by one or more
        frames of Speex data, padded to an integral number of bytes (octets)
        as specified in RFC 5574. In other words, each RTP header is replaced
        with a single byte containing the block length. Only Speex wideband is
        supported. `sample_rate_hertz` must be 16000.
      MP3: MP3 audio. MP3 encoding is a Beta feature and only available in
        v1p1beta1. Support all standard MP3 bitrates (which range from 32-320
        kbps). When using this encoding, `sample_rate_hertz` has to match the
        sample rate of the file being used.
      WEBM_OPUS: Opus encoded audio frames in WebM container
        ([OggOpus](https://wiki.xiph.org/OggOpus)). `sample_rate_hertz` must
        be one of 8000, 12000, 16000, 24000, or 48000.
    """
    ENCODING_UNSPECIFIED = 0
    LINEAR16 = 1
    FLAC = 2
    MULAW = 3
    AMR = 4
    AMR_WB = 5
    OGG_OPUS = 6
    SPEEX_WITH_HEADER_BYTE = 7
    MP3 = 8
    WEBM_OPUS = 9

  adaptation = _messages.MessageField('SpeechAdaptation', 1)
  alternativeLanguageCodes = _messages.StringField(2, repeated=True)
  audioChannelCount = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  diarizationConfig = _messages.MessageField('SpeakerDiarizationConfig', 4)
  diarizationSpeakerCount = _messages.IntegerField(5, variant=_messages.Variant.INT32)
  enableAutomaticPunctuation = _messages.BooleanField(6)
  enableSeparateRecognitionPerChannel = _messages.BooleanField(7)
  enableSpeakerDiarization = _messages.BooleanField(8)
  enableSpokenEmojis = _messages.BooleanField(9)
  enableSpokenPunctuation = _messages.BooleanField(10)
  enableWordConfidence = _messages.BooleanField(11)
  enableWordTimeOffsets = _messages.BooleanField(12)
  encoding = _messages.EnumField('EncodingValueValuesEnum', 13)
  languageCode = _messages.StringField(14)
  maxAlternatives = _messages.IntegerField(15, variant=_messages.Variant.INT32)
  metadata = _messages.MessageField('RecognitionMetadata', 16)
  model = _messages.StringField(17)
  profanityFilter = _messages.BooleanField(18)
  sampleRateHertz = _messages.IntegerField(19, variant=_messages.Variant.INT32)
  speechContexts = _messages.MessageField('SpeechContext', 20, repeated=True)
  transcriptNormalization = _messages.MessageField('TranscriptNormalization', 21)
  useEnhanced = _messages.BooleanField(22)


class RecognitionMetadata(_messages.Message):
  r"""Description of audio data to be recognized.

  Enums:
    InteractionTypeValueValuesEnum: The use case most closely describing the
      audio content to be recognized.
    MicrophoneDistanceValueValuesEnum: The audio type that most closely
      describes the audio being recognized.
    OriginalMediaTypeValueValuesEnum: The original media the speech was
      recorded on.
    RecordingDeviceTypeValueValuesEnum: The type of device the speech was
      recorded with.

  Fields:
    audioTopic: Description of the content. Eg. "Recordings of federal supreme
      court hearings from 2012".
    industryNaicsCodeOfAudio: The industry vertical to which this speech
      recognition request most closely applies. This is most indicative of the
      topics contained in the audio. Use the 6-digit NAICS code to identify
      the industry vertical - see https://www.naics.com/search/.
    interactionType: The use case most closely describing the audio content to
      be recognized.
    microphoneDistance: The audio type that most closely describes the audio
      being recognized.
    obfuscatedId: Obfuscated (privacy-protected) ID of the user, to identify
      number of unique users using the service.
    originalMediaType: The original media the speech was recorded on.
    originalMimeType: Mime type of the original audio file. For example
      `audio/m4a`, `audio/x-alaw-basic`, `audio/mp3`, `audio/3gpp`. A list of
      possible audio mime types is maintained at
      http://www.iana.org/assignments/media-types/media-types.xhtml#audio
    recordingDeviceName: The device used to make the recording. Examples
      'Nexus 5X' or 'Polycom SoundStation IP 6000' or 'POTS' or 'VoIP' or
      'Cardioid Microphone'.
    recordingDeviceType: The type of device the speech was recorded with.
  """

  class InteractionTypeValueValuesEnum(_messages.Enum):
    r"""The use case most closely describing the audio content to be
    recognized.

    Values:
      INTERACTION_TYPE_UNSPECIFIED: Use case is either unknown or is something
        other than one of the other values below.
      DISCUSSION: Multiple people in a conversation or discussion. For example
        in a meeting with two or more people actively participating. Typically
        all the primary people speaking would be in the same room (if not, see
        PHONE_CALL)
      PRESENTATION: One or more persons lecturing or presenting to others,
        mostly uninterrupted.
      PHONE_CALL: A phone-call or video-conference in which two or more
        people, who are not in the same room, are actively participating.
      VOICEMAIL: A recorded message intended for another person to listen to.
      PROFESSIONALLY_PRODUCED: Professionally produced audio (eg. TV Show,
        Podcast).
      VOICE_SEARCH: Transcribe spoken questions and queries into text.
      VOICE_COMMAND: Transcribe voice commands, such as for controlling a
        device.
      DICTATION: Transcribe speech to text to create a written document, such
        as a text-message, email or report.
    """
    INTERACTION_TYPE_UNSPECIFIED = 0
    DISCUSSION = 1
    PRESENTATION = 2
    PHONE_CALL = 3
    VOICEMAIL = 4
    PROFESSIONALLY_PRODUCED = 5
    VOICE_SEARCH = 6
    VOICE_COMMAND = 7
    DICTATION = 8

  class MicrophoneDistanceValueValuesEnum(_messages.Enum):
    r"""The audio type that most closely describes the audio being recognized.

    Values:
      MICROPHONE_DISTANCE_UNSPECIFIED: Audio type is not known.
      NEARFIELD: The audio was captured from a closely placed microphone. Eg.
        phone, dictaphone, or handheld microphone. Generally if there speaker
        is within 1 meter of the microphone.
      MIDFIELD: The speaker if within 3 meters of the microphone.
      FARFIELD: The speaker is more than 3 meters away from the microphone.
    """
    MICROPHONE_DISTANCE_UNSPECIFIED = 0
    NEARFIELD = 1
    MIDFIELD = 2
    FARFIELD = 3

  class OriginalMediaTypeValueValuesEnum(_messages.Enum):
    r"""The original media the speech was recorded on.

    Values:
      ORIGINAL_MEDIA_TYPE_UNSPECIFIED: Unknown original media type.
      AUDIO: The speech data is an audio recording.
      VIDEO: The speech data originally recorded on a video.
    """
    ORIGINAL_MEDIA_TYPE_UNSPECIFIED = 0
    AUDIO = 1
    VIDEO = 2

  class RecordingDeviceTypeValueValuesEnum(_messages.Enum):
    r"""The type of device the speech was recorded with.

    Values:
      RECORDING_DEVICE_TYPE_UNSPECIFIED: The recording device is unknown.
      SMARTPHONE: Speech was recorded on a smartphone.
      PC: Speech was recorded using a personal computer or tablet.
      PHONE_LINE: Speech was recorded over a phone line.
      VEHICLE: Speech was recorded in a vehicle.
      OTHER_OUTDOOR_DEVICE: Speech was recorded outdoors.
      OTHER_INDOOR_DEVICE: Speech was recorded indoors.
    """
    RECORDING_DEVICE_TYPE_UNSPECIFIED = 0
    SMARTPHONE = 1
    PC = 2
    PHONE_LINE = 3
    VEHICLE = 4
    OTHER_OUTDOOR_DEVICE = 5
    OTHER_INDOOR_DEVICE = 6

  audioTopic = _messages.StringField(1)
  industryNaicsCodeOfAudio = _messages.IntegerField(2, variant=_messages.Variant.UINT32)
  interactionType = _messages.EnumField('InteractionTypeValueValuesEnum', 3)
  microphoneDistance = _messages.EnumField('MicrophoneDistanceValueValuesEnum', 4)
  obfuscatedId = _messages.IntegerField(5)
  originalMediaType = _messages.EnumField('OriginalMediaTypeValueValuesEnum', 6)
  originalMimeType = _messages.StringField(7)
  recordingDeviceName = _messages.StringField(8)
  recordingDeviceType = _messages.EnumField('RecordingDeviceTypeValueValuesEnum', 9)


class RecognizeRequest(_messages.Message):
  r"""The top-level message sent by the client for the `Recognize` method.

  Fields:
    audio: Required. The audio data to be recognized.
    config: Required. Provides information to the recognizer that specifies
      how to process the request.
  """

  audio = _messages.MessageField('RecognitionAudio', 1)
  config = _messages.MessageField('RecognitionConfig', 2)


class RecognizeResponse(_messages.Message):
  r"""The only message returned to the client by the `Recognize` method. It
  contains the result as zero or more sequential `SpeechRecognitionResult`
  messages.

  Fields:
    requestId: The ID associated with the request. This is a unique ID
      specific only to the given request.
    results: Sequential list of transcription results corresponding to
      sequential portions of audio.
    totalBilledTime: When available, billed audio seconds for the
      corresponding request.
  """

  requestId = _messages.IntegerField(1)
  results = _messages.MessageField('SpeechRecognitionResult', 2, repeated=True)
  totalBilledTime = _messages.StringField(3)


class SpeakerDiarizationConfig(_messages.Message):
  r"""Config to enable speaker diarization.

  Fields:
    enableSpeakerDiarization: If 'true', enables speaker detection for each
      recognized word in the top alternative of the recognition result using a
      speaker_tag provided in the WordInfo.
    maxSpeakerCount: Maximum number of speakers in the conversation. This
      range gives you more flexibility by allowing the system to automatically
      determine the correct number of speakers. If not set, the default value
      is 6.
    minSpeakerCount: Minimum number of speakers in the conversation. This
      range gives you more flexibility by allowing the system to automatically
      determine the correct number of speakers. If not set, the default value
      is 2.
    speakerTag: Output only. Unused.
  """

  enableSpeakerDiarization = _messages.BooleanField(1)
  maxSpeakerCount = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  minSpeakerCount = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  speakerTag = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class SpeechAdaptation(_messages.Message):
  r"""Speech adaptation configuration.

  Fields:
    customClasses: A collection of custom classes. To specify the classes
      inline, leave the class' `name` blank and fill in the rest of its
      fields, giving it a unique `custom_class_id`. Refer to the inline
      defined class in phrase hints by its `custom_class_id`.
    phraseSetReferences: A collection of phrase set resource names to use.
    phraseSets: A collection of phrase sets. To specify the hints inline,
      leave the phrase set's `name` blank and fill in the rest of its fields.
      Any phrase set can use any custom class.
  """

  customClasses = _messages.MessageField('CustomClass', 1, repeated=True)
  phraseSetReferences = _messages.StringField(2, repeated=True)
  phraseSets = _messages.MessageField('PhraseSet', 3, repeated=True)


class SpeechContext(_messages.Message):
  r"""Provides "hints" to the speech recognizer to favor specific words and
  phrases in the results.

  Fields:
    boost: Hint Boost. Positive value will increase the probability that a
      specific phrase will be recognized over other similar sounding phrases.
      The higher the boost, the higher the chance of false positive
      recognition as well. Negative boost values would correspond to anti-
      biasing. Anti-biasing is not enabled, so negative boost will simply be
      ignored. Though `boost` can accept a wide range of positive values, most
      use cases are best served with values between 0 and 20. We recommend
      using a binary search approach to finding the optimal value for your use
      case.
    phrases: A list of strings containing words and phrases "hints" so that
      the speech recognition is more likely to recognize them. This can be
      used to improve the accuracy for specific words and phrases, for
      example, if specific commands are typically spoken by the user. This can
      also be used to add additional words to the vocabulary of the
      recognizer. See [usage limits](https://cloud.google.com/speech-to-
      text/quotas#content). List items can also be set to classes for groups
      of words that represent common concepts that occur in natural language.
      For example, rather than providing phrase hints for every month of the
      year, using the $MONTH class improves the likelihood of correctly
      transcribing audio that includes months.
  """

  boost = _messages.FloatField(1, variant=_messages.Variant.FLOAT)
  phrases = _messages.StringField(2, repeated=True)


class SpeechOperationsGetRequest(_messages.Message):
  r"""A SpeechOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class SpeechOperationsListRequest(_messages.Message):
  r"""A SpeechOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)


class SpeechProjectsLocationsCustomClassesCreateRequest(_messages.Message):
  r"""A SpeechProjectsLocationsCustomClassesCreateRequest object.

  Fields:
    createCustomClassRequest: A CreateCustomClassRequest resource to be passed
      as the request body.
    parent: Required. The parent resource where this custom class will be
      created. Format: `projects/{project}/locations/{location}/customClasses`
      Speech-to-Text supports three locations: `global`, `us` (US North
      America), and `eu` (Europe). If you are calling the
      `speech.googleapis.com` endpoint, use the `global` location. To specify
      a region, use a [regional endpoint](https://cloud.google.com/speech-to-
      text/docs/endpoints) with matching `us` or `eu` location value.
  """

  createCustomClassRequest = _messages.MessageField('CreateCustomClassRequest', 1)
  parent = _messages.StringField(2, required=True)


class SpeechProjectsLocationsCustomClassesDeleteRequest(_messages.Message):
  r"""A SpeechProjectsLocationsCustomClassesDeleteRequest object.

  Fields:
    name: Required. The name of the custom class to delete. Format:
      `projects/{project}/locations/{location}/customClasses/{custom_class}`
      Speech-to-Text supports three locations: `global`, `us` (US North
      America), and `eu` (Europe). If you are calling the
      `speech.googleapis.com` endpoint, use the `global` location. To specify
      a region, use a [regional endpoint](https://cloud.google.com/speech-to-
      text/docs/endpoints) with matching `us` or `eu` location value.
  """

  name = _messages.StringField(1, required=True)


class SpeechProjectsLocationsCustomClassesGetRequest(_messages.Message):
  r"""A SpeechProjectsLocationsCustomClassesGetRequest object.

  Fields:
    name: Required. The name of the custom class to retrieve. Format:
      `projects/{project}/locations/{location}/customClasses/{custom_class}`
  """

  name = _messages.StringField(1, required=True)


class SpeechProjectsLocationsCustomClassesListRequest(_messages.Message):
  r"""A SpeechProjectsLocationsCustomClassesListRequest object.

  Fields:
    pageSize: The maximum number of custom classes to return. The service may
      return fewer than this value. If unspecified, at most 50 custom classes
      will be returned. The maximum value is 1000; values above 1000 will be
      coerced to 1000.
    pageToken: A page token, received from a previous `ListCustomClass` call.
      Provide this to retrieve the subsequent page. When paginating, all other
      parameters provided to `ListCustomClass` must match the call that
      provided the page token.
    parent: Required. The parent, which owns this collection of custom
      classes. Format: `projects/{project}/locations/{location}/customClasses`
      Speech-to-Text supports three locations: `global`, `us` (US North
      America), and `eu` (Europe). If you are calling the
      `speech.googleapis.com` endpoint, use the `global` location. To specify
      a region, use a [regional endpoint](https://cloud.google.com/speech-to-
      text/docs/endpoints) with matching `us` or `eu` location value.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class SpeechProjectsLocationsCustomClassesPatchRequest(_messages.Message):
  r"""A SpeechProjectsLocationsCustomClassesPatchRequest object.

  Fields:
    customClass: A CustomClass resource to be passed as the request body.
    name: The resource name of the custom class.
    updateMask: The list of fields to be updated.
  """

  customClass = _messages.MessageField('CustomClass', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class SpeechProjectsLocationsPhraseSetsCreateRequest(_messages.Message):
  r"""A SpeechProjectsLocationsPhraseSetsCreateRequest object.

  Fields:
    createPhraseSetRequest: A CreatePhraseSetRequest resource to be passed as
      the request body.
    parent: Required. The parent resource where this phrase set will be
      created. Format: `projects/{project}/locations/{location}/phraseSets`
      Speech-to-Text supports three locations: `global`, `us` (US North
      America), and `eu` (Europe). If you are calling the
      `speech.googleapis.com` endpoint, use the `global` location. To specify
      a region, use a [regional endpoint](https://cloud.google.com/speech-to-
      text/docs/endpoints) with matching `us` or `eu` location value.
  """

  createPhraseSetRequest = _messages.MessageField('CreatePhraseSetRequest', 1)
  parent = _messages.StringField(2, required=True)


class SpeechProjectsLocationsPhraseSetsDeleteRequest(_messages.Message):
  r"""A SpeechProjectsLocationsPhraseSetsDeleteRequest object.

  Fields:
    name: Required. The name of the phrase set to delete. Format:
      `projects/{project}/locations/{location}/phraseSets/{phrase_set}`
  """

  name = _messages.StringField(1, required=True)


class SpeechProjectsLocationsPhraseSetsGetRequest(_messages.Message):
  r"""A SpeechProjectsLocationsPhraseSetsGetRequest object.

  Fields:
    name: Required. The name of the phrase set to retrieve. Format:
      `projects/{project}/locations/{location}/phraseSets/{phrase_set}`
      Speech-to-Text supports three locations: `global`, `us` (US North
      America), and `eu` (Europe). If you are calling the
      `speech.googleapis.com` endpoint, use the `global` location. To specify
      a region, use a [regional endpoint](https://cloud.google.com/speech-to-
      text/docs/endpoints) with matching `us` or `eu` location value.
  """

  name = _messages.StringField(1, required=True)


class SpeechProjectsLocationsPhraseSetsListRequest(_messages.Message):
  r"""A SpeechProjectsLocationsPhraseSetsListRequest object.

  Fields:
    pageSize: The maximum number of phrase sets to return. The service may
      return fewer than this value. If unspecified, at most 50 phrase sets
      will be returned. The maximum value is 1000; values above 1000 will be
      coerced to 1000.
    pageToken: A page token, received from a previous `ListPhraseSet` call.
      Provide this to retrieve the subsequent page. When paginating, all other
      parameters provided to `ListPhraseSet` must match the call that provided
      the page token.
    parent: Required. The parent, which owns this collection of phrase set.
      Format: `projects/{project}/locations/{location}` Speech-to-Text
      supports three locations: `global`, `us` (US North America), and `eu`
      (Europe). If you are calling the `speech.googleapis.com` endpoint, use
      the `global` location. To specify a region, use a [regional
      endpoint](https://cloud.google.com/speech-to-text/docs/endpoints) with
      matching `us` or `eu` location value.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class SpeechProjectsLocationsPhraseSetsPatchRequest(_messages.Message):
  r"""A SpeechProjectsLocationsPhraseSetsPatchRequest object.

  Fields:
    name: The resource name of the phrase set.
    phraseSet: A PhraseSet resource to be passed as the request body.
    updateMask: The list of fields to be updated.
  """

  name = _messages.StringField(1, required=True)
  phraseSet = _messages.MessageField('PhraseSet', 2)
  updateMask = _messages.StringField(3)


class SpeechRecognitionAlternative(_messages.Message):
  r"""Alternative hypotheses (a.k.a. n-best list).

  Fields:
    confidence: The confidence estimate between 0.0 and 1.0. A higher number
      indicates an estimated greater likelihood that the recognized words are
      correct. This field is set only for the top alternative of a non-
      streaming result or, of a streaming result where `is_final=true`. This
      field is not guaranteed to be accurate and users should not rely on it
      to be always provided. The default of 0.0 is a sentinel value indicating
      `confidence` was not set.
    transcript: Transcript text representing the words that the user spoke. In
      languages that use spaces to separate words, the transcript might have a
      leading space if it isn't the first result. You can concatenate each
      result to obtain the full transcript without using a separator.
    words: A list of word-specific information for each recognized word. Note:
      When `enable_speaker_diarization` is true, you will see all the words
      from the beginning of the audio.
  """

  confidence = _messages.FloatField(1, variant=_messages.Variant.FLOAT)
  transcript = _messages.StringField(2)
  words = _messages.MessageField('WordInfo', 3, repeated=True)


class SpeechRecognitionResult(_messages.Message):
  r"""A speech recognition result corresponding to a portion of the audio.

  Fields:
    alternatives: May contain one or more recognition hypotheses (up to the
      maximum specified in `max_alternatives`). These alternatives are ordered
      in terms of accuracy, with the top (first) alternative being the most
      probable, as ranked by the recognizer.
    channelTag: For multi-channel audio, this is the channel number
      corresponding to the recognized result for the audio from that channel.
      For audio_channel_count = N, its output values can range from '1' to
      'N'.
    languageCode: Output only. The [BCP-47](https://www.rfc-
      editor.org/rfc/bcp/bcp47.txt) language tag of the language in this
      result. This language code was detected to have the most likelihood of
      being spoken in the audio.
    resultEndTime: Time offset of the end of this result relative to the
      beginning of the audio.
  """

  alternatives = _messages.MessageField('SpeechRecognitionAlternative', 1, repeated=True)
  channelTag = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  languageCode = _messages.StringField(3)
  resultEndTime = _messages.StringField(4)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TranscriptNormalization(_messages.Message):
  r"""Transcription normalization configuration. Use transcription
  normalization to automatically replace parts of the transcript with phrases
  of your choosing. For StreamingRecognize, this normalization only applies to
  stable partial transcripts (stability > 0.8) and final transcripts.

  Fields:
    entries: A list of replacement entries. We will perform replacement with
      one entry at a time. For example, the second entry in ["cat" => "dog",
      "mountain cat" => "mountain dog"] will never be applied because we will
      always process the first entry before it. At most 100 entries.
  """

  entries = _messages.MessageField('Entry', 1, repeated=True)


class TranscriptOutputConfig(_messages.Message):
  r"""Specifies an optional destination for the recognition results.

  Fields:
    gcsUri: Specifies a Cloud Storage URI for the recognition results. Must be
      specified in the format: `gs://bucket_name/object_name`, and the bucket
      must already exist.
  """

  gcsUri = _messages.StringField(1)


class WordInfo(_messages.Message):
  r"""Word-specific information for recognized words.

  Fields:
    confidence: The confidence estimate between 0.0 and 1.0. A higher number
      indicates an estimated greater likelihood that the recognized words are
      correct. This field is set only for the top alternative of a non-
      streaming result or, of a streaming result where `is_final=true`. This
      field is not guaranteed to be accurate and users should not rely on it
      to be always provided. The default of 0.0 is a sentinel value indicating
      `confidence` was not set.
    endTime: Time offset relative to the beginning of the audio, and
      corresponding to the end of the spoken word. This field is only set if
      `enable_word_time_offsets=true` and only in the top hypothesis. This is
      an experimental feature and the accuracy of the time offset can vary.
    speakerTag: Output only. A distinct integer value is assigned for every
      speaker within the audio. This field specifies which one of those
      speakers was detected to have spoken this word. Value ranges from '1' to
      diarization_speaker_count. speaker_tag is set if
      enable_speaker_diarization = 'true' and only in the top alternative.
    startTime: Time offset relative to the beginning of the audio, and
      corresponding to the start of the spoken word. This field is only set if
      `enable_word_time_offsets=true` and only in the top hypothesis. This is
      an experimental feature and the accuracy of the time offset can vary.
    word: The word corresponding to this set of information.
  """

  confidence = _messages.FloatField(1, variant=_messages.Variant.FLOAT)
  endTime = _messages.StringField(2)
  speakerTag = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  startTime = _messages.StringField(4)
  word = _messages.StringField(5)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Gather stage/condition information for any important objects here."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from googlecloudsdk.core.console import progress_tracker


UPDATE_APPLICATION = 'UpdateApplication'
CREATE_DEPLOYMENT = 'CreateDeployment'
UNDEPLOY_RESOURCE = 'UndeployResource'
CLEANUP_CONFIGURATION = 'CleanupConfiguration'
_DEPLOY_STAGE_PREFIX = 'Deploy_'


def _UpdateApplicationStage(create):
  """Returns the stage for updating the Application.

  Args:
    create: whether it's for the create command.

  Returns:
    progress_tracker.Stage
  """
  if create:
    message = 'Saving Configuration for Integration...'
  else:
    message = 'Updating Configuration for Integration...'

  return progress_tracker.Stage(message, key=UPDATE_APPLICATION)


def IntegrationStages(create, resource_types):
  """Returns the progress tracker Stages for creating or updating an Integration.

  Args:
    create: whether it's for the create command.
    resource_types: set of resource type strings to deploy.

  Returns:
    dict of stage key to progress_tracker Stage.
  """

  stages = {UPDATE_APPLICATION: _UpdateApplicationStage(create)}
  stages[CREATE_DEPLOYMENT] = progress_tracker.Stage(
      'Configuring Integration...', key=CREATE_DEPLOYMENT)
  deploy_stages = _DeployStages(resource_types, 'Configuring ')
  stages.update(deploy_stages)

  return stages


# TODO(b/237396542): Add this to integration type metadata map.
def _TypeToDescriptiveName(resource_type):
  """Returns a more readable name for a resource type, for printing to console.

  Args:
    resource_type: type to be described.

  Returns:
    string with readable type name.
  """
  if resource_type == 'router':
    return 'Load Balancer'
  elif resource_type == 'service':
    return 'Cloud Run Service'
  elif resource_type == 'redis':
    return 'Redis Instance'
  elif resource_type == 'vpc':
    return 'VPC Connector'
  elif resource_type == 'cloudsql':
    return 'Cloud SQL Instance'
  return resource_type


def IntegrationDeleteStages(destroy_resource_types,
                            should_configure_service):
  """Returns the progress tracker Stages for deleting an Integration.

  Args:
    destroy_resource_types: the set of resource type strings to destroy.
    should_configure_service: bool, Whether a step to configure service binding
      is required.

  Returns:
    list of progress_tracker.Stage
  """
  stages = {}
  if should_configure_service:
    stages[UPDATE_APPLICATION] = progress_tracker.Stage(
        'Unbinding services...', key=UPDATE_APPLICATION)
    stages[CREATE_DEPLOYMENT] = progress_tracker.Stage(
        'Configuring resources...', key=CREATE_DEPLOYMENT)
    service_stages = _DeployStages({'service'}, 'Configuring ')
    stages.update(service_stages)
  stages[UNDEPLOY_RESOURCE] = progress_tracker.Stage(
      'Deleting resources...', key=UNDEPLOY_RESOURCE)
  undeploy_stages = _DeployStages(destroy_resource_types, 'Deleting ')
  stages.update(undeploy_stages)
  stages[CLEANUP_CONFIGURATION] = progress_tracker.Stage(
      'Saving Integration configurations...', key=CLEANUP_CONFIGURATION)
  return stages


def ApplyStages():
  """Returns the progress tracker Stages for apply command.

  Returns:
    array of progress_tracker.Stage
  """
  return [
      progress_tracker.Stage('Saving Configuration...', key=UPDATE_APPLICATION),
      progress_tracker.Stage(
          'Actuating Configuration...', key=CREATE_DEPLOYMENT),
  ]


def StageKeyForResourceDeployment(resource_type):
  """Returns the stage key for the step that deploys a resource type.

  Args:
    resource_type: The resource type string.

  Returns:
    stage key for deployment of type.
  """
  return _DEPLOY_STAGE_PREFIX + resource_type


def _DeployStages(resource_types, stage_prefix):
  """Appends a deploy stage for each resource type in match_type_names.

  Args:
    resource_types: The set of resource type strings in the stage.
    stage_prefix: string. The prefix to add to the stage message.

  Returns:
    dict of stage key to progress_tracker Stage.
  """
  if not resource_types:
    return {}
  stages = {}
  for resource_type in resource_types:
    message = stage_prefix + _TypeToDescriptiveName(resource_type) + '...'
    stages[StageKeyForResourceDeployment(
        resource_type)] = progress_tracker.Stage(
            message, key=StageKeyForResourceDeployment(resource_type))

  return stages



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A module for changing Cloud SDK proxy settings interactively."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import http_proxy
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.util import encoding
from googlecloudsdk.core.util import http_proxy_types

import httplib2


def ChangeGcloudProxySettings():
  """Displays proxy information and helps user set up gcloud proxy properties.

  Returns:
    Whether properties were successfully changed.
  """
  try:
    proxy_info, is_existing_proxy = EffectiveProxyInfo()
  except properties.InvalidValueError:
    log.status.Print(
        'Cloud SDK network proxy settings appear to be invalid. Proxy type, '
        'address, and port must be specified. Run [gcloud info] for more '
        'details.\n')
    is_existing_proxy = True
  else:
    _DisplayGcloudProxyInfo(proxy_info, is_existing_proxy)

  if properties.VALUES.core.disable_prompts.GetBool():
    return False

  if is_existing_proxy:
    options = ['Change Cloud SDK network proxy properties',
               'Clear all gcloud proxy properties',
               'Exit']
    existing_proxy_idx = console_io.PromptChoice(
        options, message='What would you like to do?')

    if existing_proxy_idx == 0:
      return _ProxySetupWalkthrough()

    if existing_proxy_idx == 1:
      SetGcloudProxyProperties()
      log.status.Print('Cloud SDK proxy properties cleared.\n')
      return True

    return False
  else:
    if console_io.PromptContinue(prompt_string='Do you have a network proxy '
                                 'you would like to set in gcloud'):
      return _ProxySetupWalkthrough()
    return False


def _ProxySetupWalkthrough():
  """Walks user through setting up gcloud proxy properties."""
  proxy_type_options = sorted(
      t.upper() for t in http_proxy_types.PROXY_TYPE_MAP)
  proxy_type_idx = console_io.PromptChoice(
      proxy_type_options, message='Select the proxy type:')
  if proxy_type_idx is None:
    return False
  proxy_type = proxy_type_options[proxy_type_idx].lower()

  address = console_io.PromptResponse('Enter the proxy host address: ')
  log.status.Print()
  if not address:
    return False

  port = console_io.PromptResponse('Enter the proxy port: ')
  log.status.Print()
  # Restrict port number to 0-65535.
  if not port:
    return False
  try:
    if not 0 <= int(port) <= 65535:
      log.status.Print('Port number must be between 0 and 65535.')
      return False
  except ValueError:
    log.status.Print('Please enter an integer for the port number.')
    return False

  username, password = None, None
  authenticated = console_io.PromptContinue(
      prompt_string='Is your proxy authenticated', default=False)
  if authenticated:
    username = console_io.PromptResponse('Enter the proxy username: ')
    log.status.Print()
    if not username:
      return False
    password = console_io.PromptResponse('Enter the proxy password: ')
    log.status.Print()
    if not password:
      return False

  SetGcloudProxyProperties(proxy_type=proxy_type, address=address, port=port,
                           username=username, password=password)
  log.status.Print('Cloud SDK proxy properties set.\n')
  return True


def EffectiveProxyInfo():
  """Returns ProxyInfo effective in gcloud and if it is from gloud properties.

  Returns:
    A tuple of two elements in which the first element is an httplib2.ProxyInfo
      object and the second is a bool that is True if the proxy info came from
      previously set Cloud SDK proxy properties.

  Raises:
    properties.InvalidValueError: If the properties did not include a valid set.
      "Valid" means all three of these attributes are present: proxy type, host,
      and port.
  """
  proxy_info = http_proxy.GetHttpProxyInfo()  # raises InvalidValueError
  if not proxy_info:
    return None, False

  # googlecloudsdk.core.http_proxy.GetHttpProxyInfo() will return a function
  # if there are no valid proxy settings in gcloud properties. Otherwise, it
  # will return an instantiated httplib2.ProxyInfo object.
  from_gcloud_properties = True
  if not isinstance(proxy_info, httplib2.ProxyInfo):
    from_gcloud_properties = False
    # All Google Cloud SDK network calls use https.
    proxy_info = proxy_info('https')

  return proxy_info, from_gcloud_properties


def _DisplayGcloudProxyInfo(proxy_info, from_gcloud):
  """Displays Cloud SDK proxy information."""
  if not proxy_info:
    log.status.Print()
    return

  log.status.Print('Current effective Cloud SDK network proxy settings:')
  if not from_gcloud:
    log.status.Print('(These settings are from your machine\'s environment, '
                     'not gcloud properties.)')
  proxy_type_name = http_proxy_types.REVERSE_PROXY_TYPE_MAP.get(
      proxy_info.proxy_type, 'UNKNOWN PROXY TYPE')
  log.status.Print('    type = {0}'.format(proxy_type_name))
  log.status.Print('    host = {0}'.format(proxy_info.proxy_host))
  log.status.Print('    port = {0}'.format(proxy_info.proxy_port))
  # In Python 3, httplib2 encodes the proxy username and password when
  # initializing ProxyInfo, so we want to ensure they're decoded here before
  # displaying them.
  log.status.Print('    username = {0}'.format(
      encoding.Decode(proxy_info.proxy_user)))
  log.status.Print('    password = {0}'.format(
      encoding.Decode(proxy_info.proxy_pass)))
  log.status.Print()


def SetGcloudProxyProperties(proxy_type=None, address=None, port=None,
                             username=None, password=None):
  """Sets proxy group properties; clears any property not explicitly set."""
  properties.PersistProperty(properties.VALUES.proxy.proxy_type, proxy_type)
  properties.PersistProperty(properties.VALUES.proxy.address, address)
  properties.PersistProperty(properties.VALUES.proxy.port, port)
  properties.PersistProperty(properties.VALUES.proxy.username, username)
  properties.PersistProperty(properties.VALUES.proxy.password, password)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Constants for `gcloud tasks` and `gcloud app deploy` commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import frozendict

PROJECTS_COLLECTION = 'cloudtasks.projects'
LOCATIONS_COLLECTION = 'cloudtasks.projects.locations'
QUEUES_COLLECTION = 'cloudtasks.projects.locations.queues'
TASKS_COLLECTION = 'cloudtasks.projects.locations.queues.tasks'

GCLOUD_COMMAND_ENV_KEY = 'CLOUDSDK_METRICS_COMMAND_NAME'
COMMANDS_THAT_NEED_APPENGINE = frozenset([
    'gcloud.scheduler.jobs.create.app-engine',
    'gcloud.alpha.scheduler.jobs.create.app-engine',
    'gcloud.beta.scheduler.jobs.create.app-engine',
    'gcloud.scheduler.jobs.update.app-engine',
    'gcloud.alpha.scheduler.jobs.update.app-engine',
    'gcloud.beta.scheduler.jobs.update.app-engine',
])

PULL_QUEUE = 'pull'
PUSH_QUEUE = 'push'
VALID_QUEUE_TYPES = (PULL_QUEUE, PUSH_QUEUE)

PULL_TASK = 'pull'
APP_ENGINE_TASK = 'app-engine'
HTTP_TASK = 'http'

APP_ENGINE_ROUTING_KEYS = ('service', 'version', 'instance')

APP_ENGINE_DEFAULT_LOCATION_WARNING = (
    'We are using the App Engine app location ({}) as the default location. '
    'Please use the "--location" flag if you want to use a different location.')

QUEUE_MANAGEMENT_WARNING = (
    'You are managing queues with gcloud, do not use queue.yaml or queue.xml '
    'in the future. More details at: '
    'https://cloud.google.com/tasks/docs/queue-yaml.')

MAX_RATE = 500
MAX_BUCKET_SIZE = 500

# The maximum amount of time that a task will remain in a queue without being
# executed. We use this value to have consistent behaviour with superapps's
# implementation which would have an infinite TTL as they were instead tracking
# quota usage by memory used by tasks in BigTable. The current TTL set for a
# legacy queue equals '315576000000.999999999s'.
MAX_TASK_TTL = '315360000s'  # 10 years

# The maximum amount of time that a task's name will be reserved after deletion.
# We use this value to have consistent behaviour with the legacy superapps
# implementation.
MAX_TASK_TOMBSTONE_TTL = '777600s'  # 9 days

TIME_IN_SECONDS = frozendict.frozendict({
    's': 1,
    'm': 60,
    'h': 3600,
    'd': 86400,
})

APP_TO_TASKS_ATTRIBUTES_MAPPING = frozendict.frozendict({
    'bucket_size': 'max_burst_size',
    'max_concurrent_requests': 'max_concurrent_dispatches',
    'mode': 'type',
    'name': 'name',
    'rate': 'max_dispatches_per_second',
    'retry_parameters.min_backoff_seconds': 'min_backoff',
    'retry_parameters.max_backoff_seconds': 'max_backoff',
    'retry_parameters.max_doublings': 'max_doublings',
    'retry_parameters.task_age_limit': 'max_retry_duration',
    'retry_parameters.task_retry_limit': 'max_attempts',
    'target': 'routing_override',
    # Not supported and need to deprecate if possible. See go/remove-tq-quotas
    # 'total_storage_limit': 'total_storage_limit'
})

PUSH_QUEUES_APP_DEPLOY_DEFAULT_VALUES = frozendict.frozendict({
    'max_attempts': -1,  # Translates as 'unlimited' in CT-FE
    'max_backoff': '3600s',
    'max_doublings': 16,
    'max_burst_size': 5,
    # The previous behavior when max_concurrent_dispactches was not present in
    # the YAML file was to NOT set it at all which would show up as 0 in the UI.
    # However, functionally it is no different from using the default value of
    # 1000 and this is more or less a UI fix.
    'max_concurrent_dispatches': 1000,
    'max_retry_duration': '0s',  # Translates as 'unlimited' in CT-FE
    'min_backoff': '0.100s',
})

CRON_JOB_LEGACY_DEFAULT_VALUES = frozendict.frozendict({
    'max_backoff': 3600,
    'max_doublings': 16,
    'max_retry_duration': '0s',
    'min_backoff': 0.1,
})

# Note currently CT APIs do not support modifying any pull-queue attributes
# except max_attempts and max_retry_duration while queue.yaml does not support
# max_retry_duration.
PULL_QUEUES_APP_DEPLOY_DEFAULT_VALUES = frozendict.frozendict({
    'max_attempts': -1,  # Translates as 'unlimited' in CT-FE
})



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Code that's shared between multiple org firewall policies subcommands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.resource_manager import tag_utils


def GetResourceManagerTags(resource_manager_tags):
  """Returns a map of resource manager tags, translating namespaced tags if needed.

  Args:
    resource_manager_tags: Map of resource manager tag key value pairs with
      either namespaced name or name.

  Returns:
    Map of resource manager tags with format tagKeys/[0-9]+, tagValues/[0-9]+
  """

  ret_resource_manager_tags = {}
  for key, value in resource_manager_tags.items():
    if not key.startswith('tagKeys/'):
      key = tag_utils.GetTagKeyFromNamespacedName(key).name
    if not value.startswith('tagValues/'):
      value = tag_utils.GetTagValueFromNamespacedName(value).name
    ret_resource_manager_tags[key] = value

  return ret_resource_manager_tags



"""Generated client library for billingbudgets version v1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.third_party.apis.billingbudgets.v1 import billingbudgets_v1_messages as messages


class BillingbudgetsV1(base_api.BaseApiClient):
  """Generated client library for service billingbudgets version v1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://billingbudgets.googleapis.com/'
  MTLS_BASE_URL = 'https://billingbudgets.mtls.googleapis.com/'

  _PACKAGE = 'billingbudgets'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-billing', 'https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'BillingbudgetsV1'
  _URL_VERSION = 'v1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new billingbudgets handle."""
    url = url or self.BASE_URL
    super(BillingbudgetsV1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.billingAccounts_budgets = self.BillingAccountsBudgetsService(self)
    self.billingAccounts = self.BillingAccountsService(self)

  class BillingAccountsBudgetsService(base_api.BaseApiService):
    """Service class for the billingAccounts_budgets resource."""

    _NAME = 'billingAccounts_budgets'

    def __init__(self, client):
      super(BillingbudgetsV1.BillingAccountsBudgetsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new budget. See [Quotas and limits](https://cloud.google.com/billing/quotas) for more information on the limits of the number of budgets you can create.

      Args:
        request: (BillingbudgetsBillingAccountsBudgetsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudBillingBudgetsV1Budget) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/billingAccounts/{billingAccountsId}/budgets',
        http_method='POST',
        method_id='billingbudgets.billingAccounts.budgets.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1/{+parent}/budgets',
        request_field='googleCloudBillingBudgetsV1Budget',
        request_type_name='BillingbudgetsBillingAccountsBudgetsCreateRequest',
        response_type_name='GoogleCloudBillingBudgetsV1Budget',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a budget. Returns successfully if already deleted.

      Args:
        request: (BillingbudgetsBillingAccountsBudgetsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/billingAccounts/{billingAccountsId}/budgets/{budgetsId}',
        http_method='DELETE',
        method_id='billingbudgets.billingAccounts.budgets.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='BillingbudgetsBillingAccountsBudgetsDeleteRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Returns a budget. WARNING: There are some fields exposed on the Google Cloud Console that aren't available on this API. When reading from the API, you will not see these fields in the return value, though they may have been set in the Cloud Console.

      Args:
        request: (BillingbudgetsBillingAccountsBudgetsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudBillingBudgetsV1Budget) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/billingAccounts/{billingAccountsId}/budgets/{budgetsId}',
        http_method='GET',
        method_id='billingbudgets.billingAccounts.budgets.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='BillingbudgetsBillingAccountsBudgetsGetRequest',
        response_type_name='GoogleCloudBillingBudgetsV1Budget',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Returns a list of budgets for a billing account. WARNING: There are some fields exposed on the Google Cloud Console that aren't available on this API. When reading from the API, you will not see these fields in the return value, though they may have been set in the Cloud Console.

      Args:
        request: (BillingbudgetsBillingAccountsBudgetsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudBillingBudgetsV1ListBudgetsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/billingAccounts/{billingAccountsId}/budgets',
        http_method='GET',
        method_id='billingbudgets.billingAccounts.budgets.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/budgets',
        request_field='',
        request_type_name='BillingbudgetsBillingAccountsBudgetsListRequest',
        response_type_name='GoogleCloudBillingBudgetsV1ListBudgetsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a budget and returns the updated budget. WARNING: There are some fields exposed on the Google Cloud Console that aren't available on this API. Budget fields that are not exposed in this API will not be changed by this method.

      Args:
        request: (BillingbudgetsBillingAccountsBudgetsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudBillingBudgetsV1Budget) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/billingAccounts/{billingAccountsId}/budgets/{budgetsId}',
        http_method='PATCH',
        method_id='billingbudgets.billingAccounts.budgets.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='googleCloudBillingBudgetsV1Budget',
        request_type_name='BillingbudgetsBillingAccountsBudgetsPatchRequest',
        response_type_name='GoogleCloudBillingBudgetsV1Budget',
        supports_download=False,
    )

  class BillingAccountsService(base_api.BaseApiService):
    """Service class for the billingAccounts resource."""

    _NAME = 'billingAccounts'

    def __init__(self, client):
      super(BillingbudgetsV1.BillingAccountsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""General utilities using operations in Privateca commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


from apitools.base.py import encoding
from googlecloudsdk.api_lib.privateca import base
from googlecloudsdk.api_lib.util import messages as messages_util
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import resources


class OperationError(exceptions.Error):
  """Exception for errors encountered from an operation."""


class OperationTimeoutError(OperationError):
  """Exception for when an operation times out."""


def GetOperationRef(operation):
  """Get a resource reference to a long running operation."""
  return resources.REGISTRY.ParseRelativeName(
      operation.name, 'privateca.projects.locations.operations')


def Await(operation, progress_message, api_version='v1beta1'):
  """Waits for operation to complete while displaying in-progress indicator.

  Args:
    operation: The Operation resource.
    progress_message: The message to display with the in-progress indicator.
    api_version: The API version.

  Returns:
    The resource that is the result of the operation.

  Raises:
    OperationError: if the operation did not complete successfully
  """
  if operation.done:
    if operation.error:
      raise OperationError(operation.error.message)
    return operation.response

  operation_ref = GetOperationRef(operation)
  poller = waiter.CloudOperationPollerNoResources(
      base.GetClientInstance(api_version).projects_locations_operations)
  try:
    return waiter.WaitFor(poller, operation_ref, progress_message)
  except waiter.TimeoutError:
    raise OperationTimeoutError(
        'Requested action timed out. Please run the describe command on your resource to see if changes were successful, or try again in a few minutes.'
    )


def GetMessageFromResponse(response, message_type):
  """Returns a message from the ResponseValue.

  Operations normally return a ResponseValue object in their response field that
  is somewhat difficult to use. This functions returns the corresponding
  message type to make it easier to parse the response.

  Args:
    response: The ResponseValue object that resulted from an Operation.
    message_type: The type of the message that should be returned

  Returns:
    An instance of message_type with the values from the response filled in.
  """
  message_dict = encoding.MessageToDict(response)
  # '@type' is not needed and not present in messages.
  if '@type' in message_dict:
    del message_dict['@type']
  return messages_util.DictToMessageWithErrorCheck(
      message_dict, message_type, throw_on_unexpected_fields=False)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helpers for building x509 parameters using a preset profile."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.privateca import base as privateca_base
from googlecloudsdk.api_lib.util import messages as messages_util
from googlecloudsdk.calliope import exceptions

_LEAF_CLIENT_TLS = {
    'caOptions': {
        'isCa': False
    },
    'keyUsage': {
        'extendedKeyUsage': {
            'clientAuth': True
        },
        'baseKeyUsage': {
            'digitalSignature': True,
            'keyEncipherment': True
        }
    }
}

_LEAF_CODE_SIGNING = {
    'caOptions': {
        'isCa': False
    },
    'keyUsage': {
        'extendedKeyUsage': {
            'codeSigning': True
        },
        'baseKeyUsage': {
            'digitalSignature': True,
            'contentCommitment': True
        }
    }
}

_LEAF_MTLS = {
    'caOptions': {
        'isCa': False
    },
    'keyUsage': {
        'extendedKeyUsage': {
            'serverAuth': True,
            'clientAuth': True
        },
        'baseKeyUsage': {
            'digitalSignature': True,
            'keyEncipherment': True
        }
    }
}

_LEAF_SERVER_TLS = {
    'caOptions': {
        'isCa': False
    },
    'keyUsage': {
        'extendedKeyUsage': {
            'serverAuth': True
        },
        'baseKeyUsage': {
            'digitalSignature': True,
            'keyEncipherment': True
        }
    }
}

_LEAF_SMIME = {
    'caOptions': {
        'isCa': False
    },
    'keyUsage': {
        'extendedKeyUsage': {
            'emailProtection': True
        },
        'baseKeyUsage': {
            'digitalSignature': True,
            'contentCommitment': True
        }
    }
}

_ROOT_UNCONSTRAINED = {
    'caOptions': {
        'isCa': True
    },
    'keyUsage': {
        'baseKeyUsage': {
            'certSign': True,
            'crlSign': True
        }
    }
}

_SUBORDINATE_CLIENT_TLS_PATHLEN_0 = {
    'caOptions': {
        'isCa': True,
        'maxIssuerPathLength': 0
    },
    'keyUsage': {
        'extendedKeyUsage': {
            'clientAuth': True
        },
        'baseKeyUsage': {
            'certSign': True,
            'crlSign': True
        }
    }
}

_SUBORDINATE_CODE_SIGNING_PATHLEN_0 = {
    'caOptions': {
        'isCa': True,
        'maxIssuerPathLength': 0
    },
    'keyUsage': {
        'extendedKeyUsage': {
            'codeSigning': True
        },
        'baseKeyUsage': {
            'certSign': True,
            'crlSign': True
        }
    }
}

_SUBORDINATE_MTLS_PATHLEN_0 = {
    'caOptions': {
        'isCa': True,
        'maxIssuerPathLength': 0
    },
    'keyUsage': {
        'extendedKeyUsage': {
            'serverAuth': True,
            'clientAuth': True
        },
        'baseKeyUsage': {
            'certSign': True,
            'crlSign': True
        }
    }
}

_SUBORDINATE_SERVER_TLS_PATHLEN_0 = {
    'caOptions': {
        'isCa': True,
        'maxIssuerPathLength': 0
    },
    'keyUsage': {
        'extendedKeyUsage': {
            'serverAuth': True
        },
        'baseKeyUsage': {
            'certSign': True,
            'crlSign': True
        }
    }
}

_SUBORDINATE_SMIME_PATHLEN_0 = {
    'caOptions': {
        'isCa': True,
        'maxIssuerPathLength': 0
    },
    'keyUsage': {
        'extendedKeyUsage': {
            'emailProtection': True
        },
        'baseKeyUsage': {
            'certSign': True,
            'crlSign': True
        }
    }
}

_SUBORDINATE_UNCONSTRAINED_PATHLEN_0 = {
    'caOptions': {
        'isCa': True,
        'maxIssuerPathLength': 0
    },
    'keyUsage': {
        'baseKeyUsage': {
            'certSign': True,
            'crlSign': True
        }
    }
}

_PRESET_PROFILES = {
    'leaf_client_tls': _LEAF_CLIENT_TLS,
    'leaf_code_signing': _LEAF_CODE_SIGNING,
    'leaf_mtls': _LEAF_MTLS,
    'leaf_server_tls': _LEAF_SERVER_TLS,
    'leaf_smime': _LEAF_SMIME,
    'root_unconstrained': _ROOT_UNCONSTRAINED,
    'subordinate_client_tls_pathlen_0': _SUBORDINATE_CLIENT_TLS_PATHLEN_0,
    'subordinate_code_signing_pathlen_0': _SUBORDINATE_CODE_SIGNING_PATHLEN_0,
    'subordinate_mtls_pathlen_0': _SUBORDINATE_MTLS_PATHLEN_0,
    'subordinate_server_tls_pathlen_0': _SUBORDINATE_SERVER_TLS_PATHLEN_0,
    'subordinate_smime_pathlen_0': _SUBORDINATE_SMIME_PATHLEN_0,
    'subordinate_unconstrained_pathlen_0': _SUBORDINATE_UNCONSTRAINED_PATHLEN_0
}


def GetPresetProfileOptions():
  """Returns the possible string options for the use-preset-profile flag."""
  return sorted(_PRESET_PROFILES.keys())


def GetPresetX509Parameters(profile_name):
  """Parses the profile name string into the corresponding API X509Parameters.

  Args:
    profile_name: The preset profile name.

  Returns:
    An X509Parameters object.
  """
  if profile_name not in _PRESET_PROFILES:
    raise exceptions.InvalidArgumentException(
        '--use-preset-profile',
        'The preset profile that was specified does not exist.')
  messages = privateca_base.GetMessagesModule('v1')
  return messages_util.DictToMessageWithErrorCheck(
      _PRESET_PROFILES[profile_name], messages.X509Parameters)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API client library for Certificate Manager certificates."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager

from googlecloudsdk.api_lib.certificate_manager import api_client


class CertificateClient(object):
  """API client for Certificate Manager certificates."""

  def __init__(self, client=None, messages=None):
    self._client = client or api_client.GetClientInstance()
    self._service = self._client.projects_locations_certificates
    self.messages = messages or self._client.MESSAGES_MODULE

  def Create(self,
             parent_ref,
             cert_id,
             self_managed_cert_data=None,
             description='',
             labels=None):
    """Creates a certificate.

    Args:
      parent_ref: a Resource reference to a
        certificatemanager.projects.locations resource for the parent of this
        certificate.
      cert_id: str, the ID of the cerificate to create.
      self_managed_cert_data: API message for self-managed certificate data.
      description: str, user-provided description.
      labels: Unified GCP Labels for the resource.

    Returns:
      Operation: the long running operation to create a certificate.
    """
    req = self.messages.CertificatemanagerProjectsLocationsCertificatesCreateRequest(
        parent=parent_ref.RelativeName(),
        certificateId=cert_id,
        certificate=self.messages.Certificate(
            labels=labels,
            description=description,
            selfManagedCertData=self_managed_cert_data,
        ))

    return self._service.Create(req)

  def Get(self, cert_ref):
    """Gets certificate.

    Args:
      cert_ref: a Resource reference to a
        certificatemanager.projects.locations.certificates resource to get.

    Returns:
      Certificate API representation.
    """
    get_req = self.messages.CertificatemanagerProjectsLocationsCertificatesGetRequest(
        name=cert_ref.RelativeName())
    return self._service.Get(get_req)

  def List(
      self,
      parent_ref,
      limit=None,
      page_size=None,
      list_filter=None,
      order_by=None,
  ):
    """List certificates in a given project and location.

    Args:
      parent_ref: a Resource reference to a
        certificatemanager.projects.locations resource to list certs for.
      limit: int, the total number of results to return from the API.
      page_size: int, the number of results in each batch from the API.
      list_filter: str, filter to apply in the list request.
      order_by: str, fields used for resource ordering.

    Returns:
      A list of the certificates in the project.
    """
    list_req = self.messages.CertificatemanagerProjectsLocationsCertificatesListRequest(
        parent=parent_ref.RelativeName(), filter=list_filter, orderBy=order_by)
    return list_pager.YieldFromList(
        self._service,
        list_req,
        batch_size=page_size,
        limit=limit,
        current_token_attribute='pageToken',
        next_token_attribute='nextPageToken',
        field='certificates',
        batch_size_attribute='pageSize')

  def Delete(self, cert_ref):
    """Deletes certificate.

    Args:
      cert_ref: a Resource reference to a
        certificatemanager.projects.locations.certificates resource to delete.

    Returns:
      Operation: the long running operation to delete certificate.
    """
    delete_req = self.messages.CertificatemanagerProjectsLocationsCertificatesDeleteRequest(
        name=cert_ref.RelativeName())
    return self._service.Delete(delete_req)

  def Patch(self,
            cert_ref,
            self_managed_cert_data=None,
            labels=None,
            description=None):
    """Updates a certificate.

    Used for updating labels, description and certificate data.

    Args:
      cert_ref: a Resource reference to a
        certificatemanager.projects.locations.certificates resource.
      self_managed_cert_data: API message for self-managed certificate data.
      labels: unified GCP Labels for the resource.
      description: str, new description

    Returns:
      Operation: the long running operation to patch certificate.
    """
    certificate = self.messages.Certificate()
    updated_fields = []
    if self_managed_cert_data:
      certificate.selfManaged = self_managed_cert_data
      updated_fields.append('self_managed')
    if labels:
      certificate.labels = labels
      updated_fields.append('labels')
    if description:
      certificate.description = description
      updated_fields.append('description')
    update_mask = ','.join(updated_fields)

    patch_req = self.messages.CertificatemanagerProjectsLocationsCertificatesPatchRequest(
        certificate=certificate,
        name=cert_ref.RelativeName(),
        updateMask=update_mask)
    return self._service.Patch(patch_req)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to an interface on a Distributed Cloud Edge Network router."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.edge_cloud.networking.routers import routers
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.edge_cloud.networking import resource_args
from googlecloudsdk.core import log

DESCRIPTION = (
    'Remove an interface on a Distributed Cloud Edge '
    'Network router.')
EXAMPLES = """\
    To remove the interface 'my-int-r1' on Distributed Cloud Edge Network router 'my-router' in edge zone 'us-central1-edge-den1' , run:

        $ {command} my-router --interface-name=my-int-r1 --location=us-central1 --zone=us-central1-edge-den1

   """


class RemoveInterface(base.UpdateCommand):
  """remove an interface on a Distributed Cloud Edge Network router.

  *{command}* is used to remove an interface to a Distributed Cloud Edge
  Network router.
  """
  detailed_help = {'DESCRIPTION': DESCRIPTION, 'EXAMPLES': EXAMPLES}

  @staticmethod
  def Args(parser):
    resource_args.AddRouterResourceArg(
        parser, 'from which we remove an interface', True)
    interface_parser = parser.add_mutually_exclusive_group(required=True)
    interface_parser.add_argument(
        '--interface-names',
        type=arg_parsers.ArgList(),
        metavar='INTERFACE_NAME',
        help='The list of names for interfaces being removed.')
    interface_parser.add_argument(
        '--interface-name',
        help='The name of the interface being removed.')
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    routers_client = routers.RoutersClient()
    router_ref = args.CONCEPTS.router.Parse()
    update_req_op = routers_client.RemoveInterface(router_ref, args)

    async_ = getattr(args, 'async_', False)
    if not async_:
      response = routers_client.WaitForOperation(update_req_op)
      log.UpdatedResource(router_ref, details='Operation was successful.')
      return response

    log.status.Print('Updating [{0}] with operation [{1}].'.format(
        router_ref, update_req_op.name))
